%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(function,
	  [install_function_info/0,
	   install_function_info/1,
	   function_info/5,
	   function_index/3,
	   function_name/2,
	   function_name_variant/3,
	   function_junction/2,
	   function_junction/3]).

:- use_module(swilib(info)).
:- use_module(kbset_base(person)).
:- use_module(kbset_base(textutils)).
:- use_module(kbset_base(special_words)).

:- dynamic function_info/5.
:- dynamic function_index/3.

install_function_info :-
	info(10, 'Setting up: function information'),
	retractall( function_info(_, _, _, _, _) ),
	retractall( function_index(_, _, _) ),
	( person(Id),
	  install_function_info_internal(Id),
	  fail
	; true
	).

install_function_info(Id) :-
	retractall( function_info(Id, _, _, _, _) ),
	retractall( function_index(_, _, Id) ),
	install_function_info_internal(Id).

install_function_info_internal(Id) :-
	person_function_info_de(Id, Function, Location, FromYear, ToYear),
	alnum_prefix(Location, LocationKey),
	assert( function_info(Id, Function, Location, FromYear, ToYear) ),
	assert( function_index(LocationKey, Function, Id) ).
	  

tmp_check_function_names(X) :-
	function_name_variant_de(X, Y),
	\+ function_name_de(Y).

function_name(de, X) :-
	!,
	function_name_de(X).

function_name_variant(de, X, Y) :-
	!,
	function_name_variant_de(X, Y).


:- dynamic function_de/3.
	
assert_function_de_grammar_rules :-
 	retractall(function_de(_,_,_)),
	( function_name_de(F),
	  atom_codes(F, F1),
	  expand_term((function_de(F) --> F1), C),
	  ( C = [_|_] ->
	    ( member(C1, C),
	      assert(C1),
	      fail
	    ; true
	    )
	  ; assert(C)
	  ),
	  fail
	; true
	).

:- initialization(assert_function_de_grammar_rules).

% function_name_variant_de('Pabst', 'Papst').
% function_name_variant_de('Pabstes', 'Papst').
% function_name_variant_de('Pabsts', 'Papst').
% function_name_variant_de('Papstes', 'Papst').
% function_name_variant_de('Papsts', 'Papst').
function_name_variant_de('Abtes', 'Abt').
function_name_variant_de('Abts', 'Abt').
function_name_variant_de('Bischofs', 'Bischof').
function_name_variant_de('Churfürst', 'Kurfürst').
function_name_variant_de('Churfürsten', 'Kurfürst').
function_name_variant_de('Churfürstin', 'Kurfürstin').
function_name_variant_de('Churfürsts', 'Kurfürst').
function_name_variant_de('Erzbischofes', 'Erzbischof').
function_name_variant_de('Erzbischofs', 'Erzbischof').
function_name_variant_de('Fürstbischofes', 'Fürstbischof').
function_name_variant_de('Fürstbischofs', 'Fürstbischof').
function_name_variant_de('Fürsten', 'Fürst').
function_name_variant_de('Fürsts', 'Fürst').
function_name_variant_de('Grafen', 'Graf').
function_name_variant_de('Grafs', 'Graf').
function_name_variant_de('Großherzoges', 'Großherzog').
function_name_variant_de('Großherzogs', 'Großherzog').
function_name_variant_de('Herzoges', 'Herzog').
function_name_variant_de('Herzogs', 'Herzog').
function_name_variant_de('Kaisers', 'Kaiser').
function_name_variant_de('Kardinals', 'Kardinal').
function_name_variant_de('Kurfürsten', 'Kurfürst').
function_name_variant_de('Kurfürsts', 'Kurfürst').
function_name_variant_de('Königes', 'König').
function_name_variant_de('Königs', 'König').
function_name_variant_de('Landgrafen', 'Landgraf').
function_name_variant_de('Landgrafs', 'Landgraf').
function_name_variant_de('Markgrafen', 'Markgraf').
function_name_variant_de('Markgrafs', 'Markgraf').
function_name_variant_de('Pfalzgrafen', 'Pfalzgraf').
function_name_variant_de('Pfalzgrafs', 'Pfalzgraf').
function_name_variant_de('Prinzen', 'Prinz').
function_name_variant_de('Prinzes', 'Prinz').
function_name_variant_de('Prinzs', 'Prinz').
function_name_variant_de('Titularbischofes', 'Titularbischof').
function_name_variant_de('Titularbischofs', 'Titularbischof').
function_name_variant_de('Weihbischofes', 'Weihbischof').
function_name_variant_de('Weihbischofs', 'Weihbischof').

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function_junction(de, X) :-
	!,
	function_junction_de(X).
function_junction(en, X) :-
	!,
	function_junction_en(X).

function_junction(de, X, Y) :-
	!,
	function_junction_de(X, Y).
function_junction(en, X, Y) :-
	!,
	function_junction_en(X, Y).

function_junction_de('von').
function_junction_de('zu').
function_junction_de('in').
function_junction_de('der').
function_junction_de('aus').

function_junction_en('of').
function_junction_en('from').

function_junction_de('von', 'der').
function_junction_en('of', 'the').

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

person_function_info_de(Id, Function, Location, FromYear, ToYear) :-
	findall(f(F,L,FY,TY),
		( person_extra_name(Id, _, Extra),
		  parse_function_in_name_extra(Extra,F,L,FY,TY)
 		; person_info(Id, de, BHInfo),
  		  parse_function_in_bhinfo(BHInfo,F,L,FY,TY)
		),
		Fs),
	sort(Fs, Fs1),
	member(f(Function, Location, FromYear, ToYear), Fs1).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Parse function in "Extra" part of preferred or variant name
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

parse_function_in_name_extra(Extra, F, L, Y1, Y2) :-
	atom_codes(Extra, Extra1),
	phrase(name_function_de(F, L, Y1, Y2), Extra1, _),
	!.

name_function_de(F, L, Y1, Y2) -->
	opt_items,
	opt_prefix,
	upcase_code(C),
	item(L1),
	{ atom_codes(L, [C|L1]) },
	` `,
	function_de(F),
	opt_years(Y1, Y2).

opt_prefix -->
	``.
opt_prefix -->
	`von `.
opt_prefix -->
	`zu `.
opt_prefix -->
	`zur `.
opt_prefix -->
	`zu der `.
opt_prefix -->
	`von der `.

opt_years(Y1,Y2) -->
	`, `,
	year(Y1),
	( `-`,
	  year(Y2)
        | { Y2 = Y1 }
	),
	!.
opt_years(nil,nil) -->
	\+ [_].
	
year(Y) -->
	natnum(Y),
	{ year_range(Y)
	}.

year_range(Y) :-
	Y >= 0,
	Y =< 3000,
	!.

natnum(D) -->
	digit(D0),
	digits(T),
	{ number_codes(D, [D0|T])
	}.

digit(D) --> [D], { code_type(D, digit)	}.
digits([D|T]) --> digit(D), !, digits(T).
digits([]) --> [].

item([C|Cs]) -->
	[C],
	{ C \= 0', , C \= 0'. },
	!,
	item(Cs).
item([]) -->
	[0',].

opt_items -->
	``.
opt_items -->
	opt_item,
	`, `,
	opt_items.

opt_item -->
	[C],
	{ C \= 0', },
	!,
	opt_item.
opt_item -->
	``.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Parsing gndo:biographicalOrHistoricalInformation
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% 
%%%% Nondeterministic 	
%%%%
parse_function_in_bhinfo(Bhinfo, F, L, Y1, Y2) :-
	atom_codes(Bhinfo, Bhinfo1),
	parse_function_in_bhinfo_1(Bhinfo1, F, L1, Y1, Y2),
	break_rough_location_de(L1, L),
	atom_length(L, Len),
	Len < 50.
	
parse_function_in_bhinfo_1(Bhinfo, F, L, Y1, Y2) :-
	phrase(bhinfo_function_de(F1, L1, Y11, Y21), Bhinfo, Bhinfo1),
	( F=F1, L=L1, Y1=Y11, Y2=Y21
	; parse_function_in_bhinfo_1(Bhinfo1, F, L, Y1, Y2)
	).

break_rough_location_de(X, Y) :-
	atomic_list_concat(Ys, ' und ', X),
	( Ys = [_] ->
	  atomic_list_concat(Zs, ' u. ', X),
	  ( Zs = [_] ->
	    X = Y
	  ; member(Y, Zs)
	  )
	; member(Y, Ys)
	),
	starts_with_upper_char(Y).

bhinfo_function_de(F, L, Y1, Y2) -->
	bhi_opt_items,
	function_de(F),
	bhi_junction_de,
	upcase_code(C),
	bhi_item(L1),
	bhi_opt_years(Y1, Y2),
	!,
	{ atom_codes(L, [C|L1]) }.

upcase_code(C) -->
	[C],
	{ is_upper_code(C) }.


bhi_junction_de --> ` von der `, !.
bhi_junction_de --> ` von `.
bhi_junction_de --> ` der `.
bhi_junction_de --> ` zu `.
bhi_junction_de --> ` bei `.
bhi_junction_de --> ` in der `, !.
bhi_junction_de --> ` in `.

bhi_opt_items -->
	``.
bhi_opt_items -->
	bhi_opt_item,
	(`, ` | `; `),
	!,
	bhi_opt_items.

bhi_opt_item -->
	[C],
	{ C \= 0',, C \= 0';},
	!,
	bhi_opt_item.
bhi_opt_item -->
	``.

bhi_item([C|Cs]) -->
	[C],	
	{ C \= 0',, C \= 0';,  C \= 0' , \+ code_type(C, digit) },
	!,
	bhi_item(Cs).
bhi_item([]) -->
	``.
bhi_item([0' |Cs]) -->
	` `,
	bhi_item(Cs).

bhi_opt_years(Y1,Y2) -->
	` (`,
	year(Y1),
	( `-`,
	  year(Y2)
        | { Y2 = Y1 }
	),
	`)`,
	!.
bhi_opt_years(Y1,Y2) -->
	` `,
	year(Y1),
	( `-`,
	  year(Y2)
        | { Y2 = Y1 }
	),
	!.
bhi_opt_years(Y1,nil) -->
	bhi_year_intro,
	year(Y1),
	!.
bhi_opt_years(nil,nil) -->
	` (`,
	!.
bhi_opt_years(nil,nil) -->
	`,`,
	!.
bhi_opt_years(nil,nil) -->
	`;`,
	!.
bhi_opt_years(nil,nil) -->
 	\+ [_].

bhi_year_intro --> ` ab `.
bhi_year_intro --> ` seit `.
bhi_year_intro --> ` um `.
