:- module(gnd, [ install_gnd_persons_bornbefore/1,
		 install_gnd_subjects/0,
		 gnd_setting/2,
		 gnd_subject_name/2,
		 gnd_person/1,
		 gnd_person_name/4,
		 gnd_name_to_person/4,
		 gnd_person_data/6,
		 gnd_person_data/8,
		 gnd_person_info/3,
		 gnd_to_wiki/3,
		 wiki_to_gnd/2,
		 split_gnd_name/4
	       ]).

:- use_module(swilib(err)).
:- use_module(swilib(fromonto)).
:- use_module(swilib(info)).
:- use_module(kbset_base(filename)).
:- use_module(kbset_base(utils)).
:- use_module(library(zlib)).

:- multifile gnd_setting/2.
:- multifile gnd_subject_name/2.
:- multifile gnd_person/1.
:- multifile gnd_person_name/4.
:- multifile gnd_name_to_person/4.
:- multifile gnd_person_data/6.
:- multifile gnd_person_data/8.
:- multifile gnd_person_info/3.
:- multifile gnd_to_wiki/3.
:- multifile wiki_to_gnd/2.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% gnd_person(Id).
%%%% %% differentiated person, usable to verify and enumerate
%%%% 
%%%% gnd_person_name(Id, LastName, FirstName, NameExtra).
%%%%
%%%% gnd_name_to_person(LastName, FirstName, NameExtra, Id).
%%%% 
%%%% gnd_person_data(Id, PrefGNDName, DoB, DoD, Gender, Occupations).
%%%% gnd_person_data(Id, PrefGNDName, DoB, DoD, Gender, Occupations,
%%%%                 DoBAccuracy, DoDAccuracy).
%%%%
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


install_gnd_subjects :-
	filename(gnd_subject, File),
	info(10, 'Loading ~w', [File]),
	consult(File).

install_gnd_persons_bornbefore(BornBeforeYear) :-
	filename(gnd_persons_bornbefore_pattern, F0),
	format(atom(File), F0, [BornBeforeYear]),
	info(10, 'Loading ~w', [File]),
	consult(File).

gnd_person_data(A, B, C, D, E, F) :-
	gnd_person_data(A, B, C, D, E, F, _, _).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

compose_gnd_name(Last, First, Extra, Name) :-
	( Extra = '' -> Extra1 = ''
	; atomic_list_concat([' <', Extra, '>'], Extra1)
	),
	( First = '' -> First1 = ''
	; atomic_list_concat([', ', First], First1)
	),
	atomic_list_concat([Last, First1, Extra1], Name).

split_gnd_name(type('xmls:string', Name), Last, First, Extra) :-
	!,
	split_gnd_name(Name, Last, First, Extra).
%%%% 
%%%% This worked for the 2014 GND, where the "extra" part of names was in
%%%% "<>". Now (2018) the NameEntity subobjects have to be used to access
%%%% fragments of names.
%%%%
%%%% This predicate should now only be used for manual entries that might use
%%%% the old syntax.
%%%%

% split_gnd_name(Name, Last, First, Extra) :-
% 	( sub_atom(Name, _, 1, 0, '>'),
% 	  atomic_list_concat([Name1, Extra1], ' <', Name) ->
% 	  sub_atom(Extra1, 0, _, 1, Extra),
% 	  split_gnd_last_first(Name1, Last, First)
% 	; Extra = '',
% 	  split_gnd_last_first(Name, Last, First)
% 	).
% 
% split_gnd_last_first(Name, Last, First) :-
% 	( atomic_list_concat([Last, First], ', ', Name) ->
% 	  true
% 	; First = '',
% 	  Last = Name
% 	).

split_gnd_name(Name, Last, First, Extra) :-
	( sub_atom(Name, _, 1, 0, '>'),
	  atomic_list_concat([Name1, Extra1], ' <', Name) ->
	  sub_atom(Extra1, 0, _, 1, Extra),
	  ( atomic_list_concat([Last, First], ', ', Name1) ->
	    true
	  ; Last = Name1,
	    First = ''
	  )
	; atomic_list_concat([Last, First], ', ', Name) ->
	  Extra = ''
	; atomic_list_concat([Last | Rest], ', ', Name) ->
	  atomic_list_concat(Rest, ', ', Extra),
	  First = ''
	; Last = Name,
	  First = '',
	  Extra = ''
	).

