%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(utils, [consult_datafile/1,
		  map/3,
		  map/4,
		  map/5,
		  map/6,
		  map_revbefore_after/5,
		  map_outrevbefore_after/5,
		  map_revbefore_after_seq/4,
		  map_val/2,
		  list_prefix/3,
		  list_cdr/3,
		  list_prefix_upto/3,
		  list_cdr_upto/3,
		  member_upto/3,
		  list_window_upto/4,
		  list_tail/3,
		  split_list/3,
		  map_window/6,
		  mem/3,
		  mem/2,
		  apply_append/2,
		  sum/2,
		  max/2,
		  count_up/2,
		  ord_enum/2,
		  set_of/3,
		  list_prefix_lessq/3,
		  list_prefix_less/3,
		  rkv_list_prefix_greaterq/3,
		  okv_list_prefix_lessq/3,
		  okv_list_prefix_less/3,
		  reverse_with_rest/3,
		  verify_predicates/1]).

:- module_transparent consult_datafile/1.
:- module_transparent ord_enum/2.
:- module_transparent set_of/3.

:- module_transparent map/3.
:- module_transparent map/4.
:- module_transparent map/5.
:- module_transparent map/6.

:- module_transparent map_outrevbefore_after/5.
:- module_transparent map_outrevbefore_after_1/6.
:- module_transparent moa_2/6.
:- module_transparent map_revbefore_after/5.
:- module_transparent map_revbefore_after_1/6.
:- module_transparent map_revbefore_after_seq/4.

:- use_module(swilib(info)).
:- use_module(swilib(err)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ord_enum(Term, Call) :-
	findall(Term, Call, Terms),
	sort(Terms, Terms1),
	member(Term, Terms1).
	
set_of(T, F, S) :-
	findall(T, F, S1),
	sort(S1, S).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% map_revbefore_after(P, Xs, PassedArg, Xs1) :-
% 	map_revbefore_after_1(P, Xs, PassedArg, [], Xs1).
% 
% map_revbefore_after_1(P, [X|Xs], PassedArg, RevBefore, [X1|Xs1]) :-
% 	Call =.. [P,X,PassedArg,RevBefore,Xs,X1],
% 	Call,
% 	map_revbefore_after_1(P, Xs, PassedArg, [X|RevBefore], Xs1).
% map_revbefore_after_1(_, [], _, _, []).


map_revbefore_after(P, Xs, PassedArg, PassedArg1, Xs1) :-
	map_revbefore_after_1(Xs, P, PassedArg, PassedArg1, [], Xs1).

map_revbefore_after_1([X|Xs], P, PassedArg, PassedArg1, RevBefore, [X1|Xs1]) :-
	Call =.. [P,X,PassedArg,PassedArg2,RevBefore,Xs,X1],
	Call,
	map_revbefore_after_1(Xs, P, PassedArg2, PassedArg1, [X|RevBefore], Xs1).
map_revbefore_after_1([], _, PassedArg, PassedArg, _, []).


map_revbefore_after_seq(P, [X|Xs], RevBefore, Xs1) :-
	map_revbefore_after_seq(P, [X|Xs], [], RevBefore, Xs1, []).
	
map_revbefore_after_seq(P, [X|Xs], PassedArg, RevBefore, Xs1, Xs2) :-
	Call =.. [P,X,PassedArg,RevBefore,Xs,Xs1,Xs3],
	Call,
	map_revbefore_after_seq(P, Xs, PassedArg, [X|RevBefore], Xs3, Xs2).
map_revbefore_after_seq(_, [], _, _, Xs, Xs).

% %%
% %% RevBefore are the OUTPUT elements corresponding to the preceeding list
% %% elements, implementation 1
% %%
% map_outrevbefore_after(P, Xs, PassedArg, PassedArg1, Xs1) :-
% 	map_outrevbefore_after_1(Xs, P, PassedArg, PassedArg1, [], Xs1).
% 
% map_outrevbefore_after_1([X|Xs], P, PassedArg, PassedArg1, RevBefore, [X1|Xs1]) :-
% 	Call =.. [P,X,PassedArg,PassedArg2,RevBefore,Xs,X1],
% 	Call,
% 	map_outrevbefore_after_1(Xs, P, PassedArg2, PassedArg1, [X1|RevBefore], Xs1).
% map_outrevbefore_after_1([], _, PassedArg, PassedArg, _, []).


%%
%% RevBefore are the OUTPUT elements corresponding to the preceeding list
%% elements, implementation 2
%%
map_outrevbefore_after(P, Xs, PassedArg, PassedArg1, Xs1) :-
	moa_2(Xs, P, PassedArg, PassedArg1, [], Result1),
	reverse(Result1, Xs1).

moa_2([X|Xs], P, PassedArg, PassedArg1, RevBefore, Result) :-
	Call =.. [P,X,PassedArg,PassedArg2,RevBefore,Xs,X1],
	Call,
	moa_2(Xs, P, PassedArg2, PassedArg1, [X1|RevBefore], Result).
moa_2([], _, PassedArg, PassedArg, Result, Result).

%%%% 
%%%% Examples illustrate usage:
%%%% 
% test_x1(A,B,C,D,A) :- writeln(A-B-C-D).
% 
% test_x2(A,B,C,D,[A,x|E],E) :- writeln(A-B-C-D).
% 
% text_x3 :-
% 	map_revbefore_after(test_x1,[a,b,c,d],k,X1),
% 	writeln(result(X1)),
% 	map_revbefore_after_seq(test_x2,[a,b,c,d],k,X2),
% 	writeln(result(X2)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

consult_datafile(F) :-
	( atom_concat(F1, '.pl', F) ->
	  true
	; F1 = F
	),
	info(10, 'Loading ~q', [F1]),
	consult(F1).

map(P, [X|Xs], [X1|Xs1]) :-
	Call =.. [P,X,X1],
	Call,
	map(P, Xs, Xs1).
map(_, [], []).

map(P, [X|Xs], Y, [X1|Xs1]) :-
	Call =.. [P,X,Y,X1],
	Call,
	map(P, Xs, Y, Xs1).
map(_, [], _, []).

map(P, [X|Xs], Y1, Y2, [X1|Xs1]) :-
	Call =.. [P,X,Y1,Y2,X1],
	Call,
	map(P, Xs, Y1, Y2, Xs1).
map(_, [], _, _, []).

map(P, [X|Xs], Y1, Y2, Y3, [X1|Xs1]) :-
	Call =.. [P,X,Y1,Y2,Y3,X1],
	Call,
	map(P, Xs, Y1, Y2, Y3, Xs1).
map(_, [], _, _, _, []).


map_val([_-X|Xs], [X|Xs1]) :-
	map_val(Xs, Xs1).
map_val([], []).

list_prefix(0, _, []) :-
	!.
list_prefix(N, [X|Xs], [X|Xs1]) :-
	N > 0,
	N1 is N - 1,
	list_prefix(N1, Xs, Xs1).

list_prefix_upto(0, _, []) :-
	!.
list_prefix_upto(N, [X|Xs], [X|Ys]) :-
	N > 0,
	!,
	N1 is N - 1,
	list_prefix_upto(N1, Xs, Ys).
list_prefix_upto(N, [], []) :-
	N > 0.

list_cdr(0, L, L) :-
	!.
list_cdr(N, [_|L], L1) :-
	N > 0,
	N1 is N - 1,
	list_cdr(N1, L, L1).

list_cdr_upto(0, L, L) :-
	!.
list_cdr_upto(N, [_|L], L1) :-
	N > 0,
	!,
	N1 is N - 1,
	list_cdr_upto(N1, L, L1).
list_cdr_upto(_, [], []).

list_tail(L, L, 0).
list_tail([_|L], L1, N) :-
	list_tail(L, L1, N1),
	N is N1+1.

mem(X, [X|Y], Y).
mem(X, [_|Y], Z) :-
	mem(X, Y, Z).

mem(X, Y) :-
	mem_1(Y, X).

mem_1(X, X).
mem_1([_|Y], X) :-
	mem_1(Y, X).

% mem(X, X).
% mem(X, [_|Y]) :-
% 	mem(X, Y).

apply_append([L|Ls], L1) :-
	append(L, L2, L1),
	apply_append(Ls, L2).
apply_append([], []).

sum([], 0).
sum([N|Ns], M) :- sum(Ns, N1), M is N + N1.

max([N|Ns], M) :- max(Ns, N, M).
max([N|Ns], M, N1) :- N > M, !,	max(Ns, N, N1).
max([N|Ns], M, N1) :- N =< M, max(Ns, M, N1).
max([], M, M).

count_up(To, N) :-
	count_down(To, N1),
	N is 1 + To - N1.

count_down(To, To).
count_down(To, N) :-
	To > 1,
	To1 is To-1,
	count_down(To1, N).


map_window(Xs, Pred, Arg, NBefore, NAfter, Ys) :-
	map_window(Xs, Pred, Arg, NBefore, NAfter, [], Ys).
	
map_window([X|Xs], Pred, Arg, NBefore, NAfter, RevPrev, [Y|Ys]) :-
	list_prefix_upto(NBefore, RevPrev, RevBefore),
	list_prefix_upto(NAfter, Xs, After),
	Call =.. [Pred, X, Arg, RevBefore, After, Y],
	call(Call),
	map_window(Xs, Pred, Arg, NBefore, NAfter, [X|RevPrev], Ys).
map_window([], _, _, _, _, _, []).

%%%% 
%%%% list_window_upto(+Start, +SizeUpTo, +List, -Window)
%%%% 
list_window_upto(Start, Size, List, Window) :-
	list_cdr_upto(Start, List, List1),
	list_prefix_upto(Size, List1, Window).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Assume reverse ordered input
%%%% 
rkv_list_prefix_greaterq([K-V|Xs], Y, [K-V|Xs1]) :-
	K @>= Y,
	!,
	rkv_list_prefix_greaterq(Xs, Y, Xs1).
rkv_list_prefix_greaterq(_, _, []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

okv_list_prefix_less([K-V|Xs], Y, [K-V|Xs1]) :-
	K @< Y,
	!,
	okv_list_prefix_less(Xs, Y, Xs1).
okv_list_prefix_less(_, _, []).

okv_list_prefix_lessq([K-V|Xs], Y, [K-V|Xs1]) :-
	K @=< Y,
	!,
	okv_list_prefix_lessq(Xs, Y, Xs1).
okv_list_prefix_lessq(_, _, []).

list_prefix_lessq([X|Xs], Y, [X|Xs1]) :-
	X @=< Y,
	!,
	list_prefix_lessq(Xs, Y, Xs1).
list_prefix_lessq(_, _, []).

list_prefix_less([X|Xs], Y, [X|Xs1]) :-
	X @< Y,
	!,
	list_prefix_less(Xs, Y, Xs1).
list_prefix_less(_, _, []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

reverse_with_rest(Xs, Ys, Zs) :-
	reverse_with_rest_1(Xs, Zs, Ys).

reverse_with_rest_1([], Ys, Ys).
reverse_with_rest_1([X|Xs], Rs, Ys) :-
        reverse_with_rest_1(Xs, [X|Rs], Ys).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

member_upto(B, [C|A], N) :-
	member_upto_1(A, B, C, N).

member_upto_1(_, A, A, N) :-
	N > 0.
member_upto_1([C|A], B, _, N) :-
	N > 1,
	N1 is N - 1,
	member_upto_1(A, B, C, N1).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module_transparent verify_predicates/1.

verify_predicates(PNs) :-
	( member(P/N, PNs),
	  functor(Head, P, N),
	  ( predicate_property(Head, number_of_clauses(E)) ->
	    info(10, 'Using predicate ~q/~q (~D clauses)', [P, N, E])
% 	  ; pred_dep(P/N, Call) ->
% 	    info(10, 'Calling ~q to install predicate ~q/~q', [Call,P,N]),
% 	    call(Call)
	  ; err('Missing definition of predicate ~q/~q', [P, N])
	  ),
	  fail
	; true
	).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

split_list(Xs, [], Xs).
split_list([X|Xs], [X|Xs1], Xs2) :-
	split_list(Xs, Xs1, Xs2).


	


