%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(stoplists, [is_stopword/2,
		      is_period_delimited_abbrv/2,

		      stop_de/1,
		      stop_de_18/1,
		      stop_en/1,
		      period_delimited_abbrv_de/1
		     ]).

:- use_module(swilib(err)).
:- use_module(kbset_base(textutils)).

:- encoding(utf8).

is_stopword(de, W) :-
	!,
	( stop_de(W) ->
	  true
	; downcase(W, W1),
	  W1 \= W,
	  stop_de(W1)
	).
is_stopword(de(18), W) :-
	!,
	( stop_de(W) ->
	  true
	; stop_de_18(W) ->
	  true
	; downcase(W, W1),
	  W1 \= W,
	  ( stop_de(W1) -> true
	  ; stop_de_18(W1) -> true
	  )
	).
is_stopword(de(_), W) :-
	!,
	is_stopword(de, W).
is_stopword(en, W) :-
	!,
	( stop_en(W) ->
	  true
	; downcase(W, W1),
	  W1 \= W,
	  stop_en(W1)
	).
is_stopword(en(_), W) :-
	!,
	is_stopword(en, W).

is_period_delimited_abb(de, W) :-
	!,
	( period_delimited_abbrv_de(W) ->
	  true
	; starts_with_upper_char(W),
	  downcase(W, W1),
	  period_delimited_abbrv_de(W1) ->
	  true
	).
is_period_delimited_abbrv(de(_), W) :-
	!,
	period_delimited_abbrv_de(W).


%%%%
%%%% Sources:
%%%%
%%%% The stoplist from http://snowball.tartarus.org/algorithms/de/stop.txt
%%%% and a list of frequent words from "Geschichte der Reaction", June 2014,
%%%% and other sources, see, e.g.
%%%% http://members.unine.ch/jacques.savoy/clef/index.html
%%%%
%%%% Does not include number words.
%%%%
stop_de(a).
stop_de(ab).
stop_de(aber).
stop_de(ach).
stop_de(ag).
stop_de(alle).
stop_de(allem).
stop_de(allen).
stop_de(aller).
stop_de(alles).
stop_de(als).
stop_de(also).
stop_de(am).
stop_de(an).
stop_de(ander).
stop_de(andere).
stop_de(anderem).
stop_de(anderen).
stop_de(anderer).
stop_de(anderes).
stop_de(anderm).
stop_de(andern).
stop_de(anders).
stop_de(au).
stop_de(auch).
stop_de(auf).
stop_de(aus).
stop_de(b).
stop_de(bald).
stop_de(bei).
stop_de(beim).
stop_de(bereits).
stop_de(bin).
stop_de(bis).
stop_de(bist).
stop_de(c).
stop_de(d).
stop_de(da).
stop_de(dabei).
stop_de(dadurch).
stop_de(dafür).
stop_de(dagegen).
stop_de(daher).
stop_de(dahin).
stop_de(dahinter).
stop_de(damals).
stop_de(damit).
stop_de(dank).
stop_de(dann).
stop_de(daran).
stop_de(darauf).
stop_de(darf).
stop_de(darin).
stop_de(darum).
stop_de(das).
stop_de(dass).
stop_de(dasselbe).
stop_de(davon).
stop_de(dazu).
stop_de(daß).
stop_de(dein).
stop_de(deine).
stop_de(deinem).
stop_de(deinen).
stop_de(deiner).
stop_de(deines).
stop_de(dem).
stop_de(demselben).
stop_de(den).
stop_de(denen).
stop_de(denn).
stop_de(denselben).
stop_de(der).
stop_de(deren).
stop_de(derer).
stop_de(derselbe).
stop_de(derselben).
stop_de(des).
stop_de(desselben).
stop_de(dessen).
stop_de(dich).
stop_de(die).
stop_de(dies).
stop_de(diese).
stop_de(dieselbe).
stop_de(dieselben).
stop_de(diesem).
stop_de(diesen).
stop_de(dieser).
stop_de(dieses).
stop_de(dir).
stop_de(doch).
stop_de(dort).
stop_de(drin).
stop_de(du).
stop_de(durch).
stop_de(e).
stop_de(eben).
stop_de(ei).
stop_de(ein).
stop_de(eine).
stop_de(einem).
stop_de(einen).
stop_de(einer).
stop_de(eines).
stop_de(einig).
stop_de(einige).
stop_de(einigem).
stop_de(einigen).
stop_de(einiger).
stop_de(einiges).
stop_de(einmal).
stop_de(en).
stop_de(er).
stop_de(erst).
stop_de(es).
stop_de(etwa).
stop_de(etwas).
stop_de(euch).
stop_de(euer).
stop_de(eure).
stop_de(eurem).
stop_de(euren).
stop_de(eurer).
stop_de(eures).
stop_de(f).
stop_de(für).
stop_de(g).
stop_de(gab).
stop_de(ganz).
stop_de(gar).
stop_de(geben).
stop_de(gegeben).
stop_de(gegen).
stop_de(gehabt).
stop_de(geht).
stop_de(gern).
stop_de(gewesen).
stop_de(geworden).
stop_de(gibt).
stop_de(giebt).
stop_de(ging).
stop_de(h).
stop_de(hab).
stop_de(habe).
stop_de(haben).
stop_de(habt).
stop_de(hast).
stop_de(hat).
stop_de(hatte).
stop_de(hatten).
stop_de(her).
stop_de(hier).
stop_de(hin).
stop_de(hinter).
stop_de(hätte).
stop_de(hätten).
stop_de(i).
stop_de(ich).
stop_de(ihm).
stop_de(ihn).
stop_de(ihnen).
stop_de(ihr).
stop_de(ihre).
stop_de(ihrem).
stop_de(ihren).
stop_de(ihrer).
stop_de(ihres).
stop_de(im).
stop_de(in).
stop_de(indem).
stop_de(ins).
stop_de(ist).
stop_de(j).
stop_de(ja).
stop_de(je).
stop_de(jede).
stop_de(jedem).
stop_de(jeden).
stop_de(jeder).
stop_de(jedes).
stop_de(jene).
stop_de(jenem).
stop_de(jenen).
stop_de(jener).
stop_de(jenes).
stop_de(jetzt).
stop_de(k).
stop_de(kam).
stop_de(kann).
stop_de(kein).
stop_de(keine).
stop_de(keinem).
stop_de(keinen).
stop_de(keiner).
stop_de(keines).
stop_de(konnte).
stop_de(können).
stop_de(könnte).
stop_de(l).
stop_de(m).
stop_de(machen).
stop_de(mag).
stop_de(mahn).
stop_de(man).
stop_de(manche).
stop_de(manchem).
stop_de(manchen).
stop_de(mancher).
stop_de(manches).
stop_de(mann).
stop_de(mein).
stop_de(meine).
stop_de(meinem).
stop_de(meinen).
stop_de(meiner).
stop_de(meines).
stop_de(mich).
stop_de(mir).
stop_de(mit).
stop_de(mittel).
stop_de(muss).
stop_de(musste).
stop_de(muß).
stop_de(mußte).
stop_de(mögt).
stop_de(müssen).
stop_de(n).
stop_de(na).
stop_de(nach).
stop_de(nahm).
stop_de(nein).
stop_de(neue).
stop_de(nicht).
stop_de(nichts).
stop_de(nie).
stop_de(noch).
stop_de(nun).
stop_de(nur).
stop_de(o).
stop_de(ob).
stop_de(oben).
stop_de(oder).
stop_de(oft).
stop_de(ohne).
stop_de(p).
stop_de(q).
stop_de(r).
stop_de(rund).
stop_de(s).
stop_de(sa).
stop_de(sagt).
stop_de(sah).
stop_de(satt).
stop_de(schon).
stop_de(sehr).
stop_de(sei).
stop_de(seid).
stop_de(sein).
stop_de(seine).
stop_de(seinem).
stop_de(seinen).
stop_de(seiner).
stop_de(seines).
stop_de(selbst).
stop_de(sich).
stop_de(sie).
stop_de(sind).
stop_de(so).
stop_de(solche).
stop_de(solchem).
stop_de(solchen).
stop_de(solcher).
stop_de(solches).
stop_de(soll).
stop_de(sollen).
stop_de(sollte).
stop_de(sondern).
stop_de(sonst).
stop_de(sowie).
stop_de(t).
stop_de(tat).
stop_de(tun).
stop_de(u).
stop_de(ueber).
stop_de(uhr).
stop_de(um).
stop_de(und).
stop_de(uns).
stop_de(unse).
stop_de(unsem).
stop_de(unsen).
stop_de(unser).
stop_de(unses).
stop_de(unter).
stop_de(v).
stop_de(viel).
stop_de(vom).
stop_de(von).
stop_de(vor).
stop_de(w).
stop_de(war).
stop_de(waren).
stop_de(warst).
stop_de(wart).
stop_de(was).
stop_de(weg).
stop_de(weil).
stop_de(weiter).
stop_de(welche).
stop_de(welchem).
stop_de(welchen).
stop_de(welcher).
stop_de(welches).
stop_de(wem).
stop_de(wen).
stop_de(wenigstens).
stop_de(wenn).
stop_de(wer).
stop_de(werde).
stop_de(werden).
stop_de(werdet).
stop_de(wessen).
stop_de(wie).
stop_de(wieder).
stop_de(will).
stop_de(wir).
stop_de(wird).
stop_de(wirst).
stop_de(wo).
stop_de(wollen).
stop_de(wollte).
stop_de(worden).
stop_de(wurde).
stop_de(während).
stop_de(wäre).
stop_de(würde).
stop_de(würden).
stop_de(x).
stop_de(y).
stop_de(z).
stop_de(zu).
stop_de(zum).
stop_de(zur).
stop_de(zwar).
stop_de(zwischen).
stop_de(über).


%%%% 
%%%% Frequent German words in the 18th century that are not included in
%%%% stop_de.  Source: Frequent words in the transcription of the
%%%% Sulzer/Bodmer letters.
%%%% 

stop_de_18(bey).
stop_de_18(dabey).
stop_de_18(dise).
stop_de_18(diser).
stop_de_18(disem).
stop_de_18(disen).
stop_de_18(dises).
stop_de_18(gemacht).
stop_de_18(hr).
stop_de_18(hrn).
stop_de_18(iezo).
stop_de_18(izt).
stop_de_18(kommen).
stop_de_18(könne).
stop_de_18(macht).
stop_de_18(mehr).
stop_de_18(müßen).
stop_de_18(recht).
stop_de_18(sagen).
stop_de_18(sey).
stop_de_18(seyn).
stop_de_18(thun).
stop_de_18(unsern).
stop_de_18(vil).
stop_de_18(wol).

%%%%
%%%% Basic Source:
%%%% http://snowball.tartarus.org/algorithms/en/stop.txt
%%%%
%%%% Does not include number words.
%%%%
stop_en(a).
stop_en(about).
stop_en(above).
stop_en(after).
stop_en(again).
stop_en(against).
stop_en(all).
stop_en(am).
stop_en(an).
stop_en(and).
stop_en(any).
stop_en(are).
stop_en('aren\'t').
stop_en(as).
stop_en(at).
stop_en(b).
stop_en(be).
stop_en(because).
stop_en(been).
stop_en(before).
stop_en(being).
stop_en(below).
stop_en(between).
stop_en(both).
stop_en(but).
stop_en(by).
stop_en(c).
stop_en('can\'t').
stop_en(cannot).
stop_en(could).
stop_en('couldn\'t').
stop_en(d).
stop_en(did).
stop_en('didn\'t').
stop_en(do).
stop_en(does).
stop_en('doesn\'t').
stop_en(doing).
stop_en('don\'t').
stop_en(down).
stop_en(during).
stop_en(e).
stop_en(each).
stop_en(f).
stop_en(few).
stop_en(for).
stop_en(from).
stop_en(further).
stop_en(g).
stop_en(h).
stop_en(had).
stop_en('hadn\'t').
stop_en(has).
stop_en('hasn\'t').
stop_en(have).
stop_en('haven\'t').
stop_en(having).
stop_en(he).
stop_en('he\'d').
stop_en('he\'ll').
stop_en('he\'s').
stop_en(her).
stop_en(here).
stop_en('here\'s').
stop_en(hers).
stop_en(herself).
stop_en(him).
stop_en(himself).
stop_en(his).
stop_en(how).
stop_en('how\'s').
stop_en(i).
stop_en('i\'d').
stop_en('i\'ll').
stop_en('i\'m').
stop_en('i\'ve').
stop_en(if).
stop_en(in).
stop_en(into).
stop_en(is).
stop_en('isn\'t').
stop_en(it).
stop_en('it\'s').
stop_en(its).
stop_en(itself).
stop_en(j).
stop_en(k).
stop_en(l).
stop_en('let\'s').
stop_en(m).
stop_en(me).
stop_en(more).
stop_en(most).
stop_en('mustn\'t').
stop_en(my).
stop_en(myself).
stop_en(n).
stop_en(no).
stop_en(nor).
stop_en(not).
stop_en(o).
stop_en(of).
stop_en(off).
stop_en(on).
stop_en(once).
stop_en(only).
stop_en(or).
stop_en(other).
stop_en(ought).
stop_en(our).
stop_en(ours).
stop_en(ourselves).
stop_en(out).
stop_en(over).
stop_en(own).
stop_en(p).
stop_en(q).
stop_en(r).
stop_en(s).
stop_en(same).
stop_en('shan\'t').
stop_en(she).
stop_en('she\'d').
stop_en('she\'ll').
stop_en('she\'s').
stop_en(should).
stop_en('shouldn\'t').
stop_en(so).
stop_en(some).
stop_en(such).
stop_en(t).
stop_en(than).
stop_en(that).
stop_en('that\'s').
stop_en(the).
stop_en(their).
stop_en(theirs).
stop_en(them).
stop_en(themselves).
stop_en(then).
stop_en(there).
stop_en('there\'s').
stop_en(these).
stop_en(they).
stop_en('they\'d').
stop_en('they\'ll').
stop_en('they\'re').
stop_en('they\'ve').
stop_en(this).
stop_en(those).
stop_en(through).
stop_en(to).
stop_en(too).
stop_en(u).
stop_en(under).
stop_en(until).
stop_en(up).
stop_en(v).
stop_en(very).
stop_en(w).
stop_en(was).
stop_en('wasn\'t').
stop_en(we).
stop_en('we\'d').
stop_en('we\'ll').
stop_en('we\'re').
stop_en('we\'ve').
stop_en(were).
stop_en('weren\'t').
stop_en(what).
stop_en('what\'s').
stop_en(when).
stop_en('when\'s').
stop_en(where).
stop_en('where\'s').
stop_en(which).
stop_en(while).
stop_en(who).
stop_en('who\'s').
stop_en(whom).
stop_en(why).
stop_en('why\'s').
stop_en(with).
stop_en('won\'t').
stop_en(would).
stop_en('wouldn\'t').
stop_en(x).
stop_en(y).
stop_en(you).
stop_en('you\'d').
stop_en('you\'ll').
stop_en('you\'re').
stop_en('you\'ve').
stop_en(your).
stop_en(yours).
stop_en(yourself).
stop_en(yourselves).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


period_delimited_abbrv_de('Apr').
period_delimited_abbrv_de('Aug').
period_delimited_abbrv_de('Ch').
period_delimited_abbrv_de('Dec').
period_delimited_abbrv_de('Dez').
period_delimited_abbrv_de('Di').
period_delimited_abbrv_de('Dipl').
period_delimited_abbrv_de('Do').
period_delimited_abbrv_de('Dr').
period_delimited_abbrv_de('Ew').
period_delimited_abbrv_de('Feb').
period_delimited_abbrv_de('Fr').
period_delimited_abbrv_de('Frl').
period_delimited_abbrv_de('Gr').
period_delimited_abbrv_de('Ing').
period_delimited_abbrv_de('Jan').
period_delimited_abbrv_de('Jul').
period_delimited_abbrv_de('Jun').
period_delimited_abbrv_de('Kl').
period_delimited_abbrv_de('Maj').
period_delimited_abbrv_de('Mar').
period_delimited_abbrv_de('Mi').
period_delimited_abbrv_de('Mo').
period_delimited_abbrv_de('Mr').
period_delimited_abbrv_de('Mrs').
period_delimited_abbrv_de('Mär').
period_delimited_abbrv_de('Nov').
period_delimited_abbrv_de('Nr').
period_delimited_abbrv_de('Oct').
period_delimited_abbrv_de('Okt').
period_delimited_abbrv_de('Prof').
period_delimited_abbrv_de('Ps').
period_delimited_abbrv_de('Sa').
period_delimited_abbrv_de('Sep').
period_delimited_abbrv_de('So').
period_delimited_abbrv_de('Sr').
period_delimited_abbrv_de('St').
period_delimited_abbrv_de('Str').
period_delimited_abbrv_de('Tel').
period_delimited_abbrv_de('Tsnd').
period_delimited_abbrv_de('Zt').
period_delimited_abbrv_de('abg').
period_delimited_abbrv_de('abs').
period_delimited_abbrv_de('bd').
period_delimited_abbrv_de('betr').
period_delimited_abbrv_de('bzw').
period_delimited_abbrv_de('ca').
period_delimited_abbrv_de('dt').
period_delimited_abbrv_de('etc').
period_delimited_abbrv_de('ev').
period_delimited_abbrv_de('evtl').
period_delimited_abbrv_de('geogr').
period_delimited_abbrv_de('ggf').
period_delimited_abbrv_de('gr').
period_delimited_abbrv_de('insz').
period_delimited_abbrv_de('kath').
period_delimited_abbrv_de('lfd').
period_delimited_abbrv_de('med').
period_delimited_abbrv_de('mrd').
period_delimited_abbrv_de('nat').
period_delimited_abbrv_de('nr').
period_delimited_abbrv_de('usf').
period_delimited_abbrv_de('usw').
period_delimited_abbrv_de('vgl').
period_delimited_abbrv_de('vs').
