%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module( data_interface,
	   [enum_work/1,
	    work_author/2,
	    work_title/2,
	    
	    enum_xperson/1,
	    xperson_prettyname/2,
	    xperson_furthernames/2,
	    xperson_birth_normdate/2,
	    xperson_death_normdate/2,
	    xperson_links/2,
	    xperson_gender/2,
	    xperson_def/2,
	    xperson_text/2,
	    
	    enum_geo/1,
	    geo_ename/2,
	    geo_subgeos/2,
	    geo_supergeo/2,
	    geo_furthernames/2,
	    geo_links/2,

	    enum_event/1,
	    event_ename/2,
	    event_subevents/2,
	    event_superevent/2,
% 	    event_sortdate/2,
% 	    event_prettydate/2,
	    event_links/2,

	    enum_subject/1,
	    subject_ename/2,
	    enum_journal/1,
	    journal_ename/2,
	    enum_corp/1,
	    corp_ename/2,

	    headername/2,
	    
	    img/5,
	    aut/5,
	    aut_license/2,

	    bibrefdata/5,

	    stdize_id/2,
	    
	    refresh_access_data/0
	    ]
	 ).



/** _ename for "entity name", because
  some _name predicates are already used by the KBSET core
*/

:- use_module(swilib(info)).
:- use_module(kbset_letters(match_atom)).
:- use_module(kbset_letters(letters_config)).
:- use_module(kbset_base(filename)).

:- multifile autograph/5.
:- dynamic bibrefdata/5.
:- multifile geohack/2.
:- multifile gnd_dewiki/2.
:- multifile gnd_person/7.
:- multifile person/7.
:- multifile image/5.
:- multifile person_to_gnd/2.
:- multifile person_to_dewiki/2.

:- multifile defcorp/2.
:- multifile defevent/2.
:- multifile defgeo/2.
:- multifile defsubgeo/3.
:- multifile defjournal/2.
:- multifile defperson/2.
:- multifile defsubevent/3.
:- multifile defsubject/2.
:- multifile defwork/3.
:- multifile defworkTwo/4.
:- multifile defworkThree/5.

:- multifile defcorpSee/2.
:- multifile defeventSee/2.
:- multifile defgeoSee/2.
:- multifile defjournalSee/2.
:- multifile defpersonSee/2.
:- multifile defsubjectSee/2.

:- multifile defheadername/2.

:- dynamic xperson_text/2.
:- dynamic person_text/2.

refresh_access_data :-
	info(10, 'Refreshing access data'),
	ensure_letters_initialized,
	get_kbset_config(factbase_latex_sources, DBs),
	findall( gendata(File),
		 ( member(DB, DBs),
		   format(atom(File), 'generated_from_latexdb_~w.pl', DB) ),
		 Files1),
	get_kbset_config(data_sources, DBs1),
	findall( appdata(File1), member(File1, DBs1), Files2),
	append(Files1, Files2, Files3),
	Files = [ %% TODO: replace with a more general factbase (not just
	          %% German, not just until 1780) or pass this to config
		  kbset_data(gnd_to_dewiki_1780) | Files3 ],
	( member(F, Files),
	  filename( F, F1 ),
	  load_appdata_file(F1),
	  fail
	; true
	),
	filename( gendata(generated_bibrefdata), FBibRef ),
	atom_concat(FBibRef, '.pl', FBibRefPL),
	( exists_file(FBibRefPL) ->
	  info(10, 'Consulting ~w', [FBibRef]),
	  consult(FBibRef)
	; info(10, 'WARNING: Working without existing generated bibrefdata file')
	),
	prepare_person_data.

load_appdata_file(F) :-
	( sub_atom(F, _, _, 0, '.pl') ->
	  F1 = F,
	  Mode = prolog
	; sub_atom(F, _, _, 0, '.html') ->
	  F1 = F,
	  Mode = html
	; atom_concat(F, '.pl', F2), exists_file(F2) ->
	  F1 = F,
	  Mode = prolog
	; atom_concat(F, '.html', F2), exists_file(F2) ->
	  F1 = F2,
	  Mode = html
	),
	( Mode = prolog ->
	  info(10, 'Consulting ~w', [F1]),
	  consult(F1)
	; Mode = html ->
	  info(10, 'Loading HTML data file ~w', [F1]),
	  load_html_appdata_file(F1)
	).

load_html_appdata_file(F1) :-
	load_html(F1, Es, [dialect(html5)]),
	( sub_term(element(div, Atts, Value), Es),
	  memberchk(class=Class, Atts),
	  atom_prefix(Class, 'xentry-'),
	  memberchk(id=Id, Atts),
	  atom_concat('xentry-', Predicate, Class),
	  Fact =.. [Predicate, Id, Value1],
	  retractall( Fact ),
	  Value1 = Value,
	  assertz( Fact ),
	  fail
	; true
	).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% bibrefdata1(A, B, C, D, E) :-
% 	bibrefdata(A, B, C, D, E).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Works (Werke)
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

enum_work(X) :-
	( defwork(X, _, _)
	; defworkTwo(X, _, _, _)
	; defworkThree(X, _, _, _, _)
	).

work_author(X, Y) :- defwork(X, Y, _).
work_author(X, Y) :- defworkTwo(X, Y, _, _).
work_author(X, Y) :- defworkTwo(X, _, Y, _).
work_author(X, Y) :- defworkThree(X, Y, _, _, _).
work_author(X, Y) :- defworkThree(X, _, Y, _, _).
work_author(X, Y) :- defworkThree(X, _, _, Y, _).
%%%%
work_title(X, Y) :- defwork(X, _, Y).
work_title(X, Y) :- defworkTwo(X, _, _, Y).
work_title(X, Y) :- defworkThree(X, _, _, _, Y).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Persons
%%%% 
%%%% "xperson_" instead of "person_" to allow co-existence with other kbset
%%%% modules
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- dynamic person2/6.

enum_xperson(P) :-
	findall(X, person1(X, _, _, _, _, _, _), Xs),
	sort(Xs, Xs1),
	member(P, Xs1).

person1(A, B, C, D, E, F, G) :-
	person(A, B, C, D, E, F, G).
person1(A, B, H, unknown, E, F, G) :-
	person2(A, B, E, F, G, H).

prepare_person_data :-
	info(10, 'Preparing person data'),
	retractall( person2(_, _, _, _, _, _) ),
	( defperson(A, Person),
	  \+ person(A, _, _, _, _, _, _),
	  split_defperson_entry(Person, B, E, F),
	  compute_xperson_links(A, G),
	  compute_xperson_furthernames(A, H),
	  assert( person2(A, B, E, F, G, H) ),
	  fail
	; true
	),
	( retractall( xperson_text(_, _) ),
	  enum_xperson(A),
	  stdize_id(A, A1),
	  person_text(A1, T),
	  assertz( xperson_text(A, T) ),
	  fail
	; true
	).
	  
compute_xperson_links(X, Links) :-
	( person_to_gnd(X, GND) ->
	  Links = [gnd-GND|Links1],
	  ( gnd_dewiki(GND, WikiDe) ->
	    Links1 = [wiki_de-WikiDe]
	  ; person_to_dewiki(X, WikiDe) ->
	    Links1 = [wiki_de-WikiDe]
	  ; Links1 = []
	  )
	; person_to_dewiki(X, WikiDe) ->
	  Links = [wiki_de-WikiDe]
	; Links = []
	).

compute_xperson_furthernames(X, Names) :-
	findall(N, defpersonSee(X, N), Names).

split_defperson_entry(Person, Name, YB, YD) :-
	match_atom([Name1, ' (', _, ')', Rest1, ', ', Rest2], Person),
	!,
	atomic_list_concat([Name1, Rest1, ', ', Rest2], Person1),
	split_defperson_entry(Person1, Name, YB, YD).
split_defperson_entry(Person, Name, YB, YD) :-
	match_atom([Name, ' (', YB1, '--', YD1, ' v. Chr.)'], Person),
	!,
	atom_concat('-', YB1, YB),
	atom_concat('-', YD1, YD).
split_defperson_entry(Person, Name, YB, YD) :-
	match_atom([Name, ' (', YB1, ' v. Chr. --', YD,')'], Person),
	!,
	atom_concat('-', YB1, YB).
split_defperson_entry(Person, Name, YB, YD) :-
	match_atom([Name, ' (', YB, '--', YD, ')'], Person),
	!.
split_defperson_entry(Person, Person, '?', '?').

xperson_prettyname(P, X) :-     person1(P, X, _, _, _, _, _), !.
xperson_furthernames(P, X) :-   person1(P, _, X, _, _, _, _), !.
xperson_gender(P, X) :-         person1(P, _, _, X, _, _, _), !.
xperson_links(P, X) :-          person1(P, _, _, _, _, _, X), !.
xperson_birth_normdate(P, X) :-
	person1(P, _, F, _, X1, _, _),
	( X1 = unknown, person_fwd(F, P1) ->
	  person1(P1, _, _, _, X, _, _)
	; X1 = X
	).
xperson_death_normdate(P, X) :-
	person1(P, _, F, _, _, X1, _),
	( X1 = unknown, person_fwd(F, P1) ->
	  person1(P1, _, _, _, _, X, _)
	; X1 = X
	).

person_fwd(Furthernames, P1) :-
	memberchk(pseudonym_of-P1, Furthernames),
	!.

xperson_def(P, N) :-
	enum_xperson(P),
	xperson_prettyname(P, N1),
	xperson_birth_normdate(P, D1),
	xperson_death_normdate(P, D2),
	( D1 = unknown -> D11 = '????' ; D11 = D1),
	( D2 = unknown ->
	  ( D1 = unknown -> D21 = '????' ; D21 = '' )
	; D21 = D2
	),
	format(atom(N), '~w (~w--~w)', [N1, D11, D21]).
	


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Images, Autographs
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

img(A,B,C,D,E) :-
	image(A,B,C,D,E).
	
aut(A,B,C,D,E) :-
	autograph(A,B,C,D,E).

aut_license(A, B) :-
	aut(_,_,_,A,B).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Geo
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

enum_geo(X) :-
	( defgeo(X, _)
	; defsubgeo(X, _, _)
	).

geo_ename(X, Y) :-
	( defgeo(X, Y) -> true
	; defsubgeo(X, _, Y) -> true
	).

geo_subgeos(X, Y) :-
	findall(Z, defsubgeo(Z, X, _), Y).

geo_supergeo(X, Y) :-
	defsubgeo(X, Y, _).

geo_furthernames(_, []).

geo_links(X, [LinkType-URL]) :-
	geohack(X, URL1),
	!,
	( sub_atom(URL1, 0, E, _, 'http://en.wikipedia.org/wiki/') ->
	  LinkType = wiki_en,
	  sub_atom(URL1, E, _, 0, URL)
	; sub_atom(URL1, 0, E, _, 'http://de.wikipedia.org/wiki/') ->
	  LinkType = wiki_de,
	  sub_atom(URL1, E, _, 0, URL)
	; sub_atom(URL1, 0, E, _, 'https://tools.wmflabs.org/geohack/geohack.php?language=de&') ->
	  LinkType = geohack_de,
	  sub_atom(URL1, E, _, 0, URL)
	; LinkType = unknown,
	  URL1 = URL
	).
geo_links(_, []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

enum_subject(X) :-
	defsubject(X, _).

subject_ename(X, Y) :-
	defsubject(X, Y).

enum_corp(X) :-
	defcorp(X, _).

corp_ename(X, Y) :-
	defcorp(X, Y).

enum_journal(X) :-
	defjournal(X, _).

journal_ename(X, Y) :-
	defjournal(X, Y).

headername(X, Y) :-
	defheadername(X, Y).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

enum_event(X) :-
	( defevent(X, _)
	; defsubevent(X, _, _)
	).

event_ename(X, Y) :-
	( defevent(X, Y) -> true
	; defsubevent(X, _, Y) -> true
	).
event_subevents(X, Y) :-
	findall(Z, defsubevent(Z, X, _), Y).
event_superevent(X, Y) :-
	defsubevent(X, Y, _).
event_links(_, []).
% event_prettydate(X, Y) :-
% 	tmp_e_pd(X, Y),
% 	!.
% event_prettydate(_, '(Datum)').
% event_sortdate(X, Y) :-
% 	tmp_e_sd(X, Y),
% 	!.
% event_sortdate(_, '1700/01/01').

%%%% 
%%%% Note: Windows does not accept ":" in filenames
%%%% 

stdize_id(ID, ID1) :-
	downcase_atom(ID, ID2),
	atom_chars(ID2, ID3),
	map_fix_special_char(ID3, ID4),
	atom_chars(ID1, ID4).

map_fix_special_char([X|Xs], [X1|Xs1]) :-
	fix_special_char(X, X1),
	map_fix_special_char(Xs, Xs1).
map_fix_special_char([], []).

fix_special_char(X, '-') :- \+ char_type(X, ascii), !.
fix_special_char(X, '-') :- \+ char_type(X, alnum), !.
fix_special_char(X, X).
