%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(geo_handling, [geo_sortkey/2]).

:- use_module(kbset_letters(textutilities)).
:- use_module(swilib(err)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Ort
%%%% 	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%% 
%%%% Ort: Parsing
%%%% 	

text_geo(T, O) :-
	atom_codes(T, S),
	phrase(geo(O), S, []),
	!.
text_geo(T, _) :-
	err('Ort kann nicht gelesen werden: ~q', [T]).

geo(O) -->
	coregeo(O).
	
coregeo(ohne_ort) -->
	`ohne Ort`,
	!.	
coregeo(ohne_ort) -->
	`o. O.`,
	!.	
coregeo(geo(O)) -->
	ortsname(O).
coregeo(geo_vermutlich(O)) -->
	`[`,
	ortsname(O),
	`]`.

ortsname(W) -->
	alpha(A),
	alphas(As),
	{ atom_codes(W, [A|As])
	}.

alpha(A) -->
	[A],
	{ memberchk(A, `äöüÄÖÜß/-() `)
	},
	!.
alpha(A) -->
	[A],
	{ code_type(A, alpha)
	},
	!.

alphas([A|B]) -->
	alpha(A), !,
	alphas(B).
alphas([]) -->
	[].


%%%% 
%%%% Ort: Pretty Printing
%%%%
geo_pretty(O, P) :-
	with_output_to(atom(P), pp_geo(O)).

pp_geo(ohne_ort) :-
	!,
	format('ohne Ort', []).
pp_geo(geo(Ort)) :-
	!,
	format('~w', [Ort]).
pp_geo(geo_vermutlich(Ort)) :-
	!,
	format('[~w]', [Ort]).
pp_geo(X) :-
	err('Ort kann nicht gedruckt werden: ~q', [X]).

%%%% 
%%%% Ort: Key for Sorting and use as ID
%%%%
geo_sortkey(geo(Name), K) :-
	!,
	atom_to_ascii(Name, K1),
	downcase_atom(K1, K).
geo_sortkey(vermutlich(Name), K) :-
	!,
	atom_to_ascii(Name, K1),
	downcase_atom(K1, K).
geo_sortkey(_, z(ohne_ort)).
