%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(name_conversions, [short_person_name/2]).

:- use_module(match_atom).

short_person_name(Name, Name1) :-
	match_atom([N, ', ', R], Name),
	!,
	( sub_atom(N, _, _, _, ' ') ->
	  atom_chars(N, N1),
	  reverse(N1, N2),
	  ( rev_num_postfix(N2) ->
	    Name1 = N
	  ; initials(R, R1),
	    atomic_list_concat([N, ', ', R1], Name1)
	  )
	; sub_atom(R, _, _, _, ',') ->
	  Name1 = N 
	; initials(R, R1),
	  atomic_list_concat([N, ', ', R1], Name1)
	).
short_person_name(Name, Name).

rev_num_postfix(['.'|_]) :- !.
rev_num_postfix([X|_]) :- memberchk(X, ['I', 'V', 'X', 'C']).

initials(S, S1) :-
	atomic_list_concat(Parts, ' ', S),
	map_initial(Parts, Parts1),
	atomic_list_concat(Parts1, ' ', S1).

map_initial([X|Xs], [X1|Xs1]) :-
	initial(X, X1),
	map_initial(Xs, Xs1).
map_initial([], []).

initial(de, de) :-
	!.
initial(X, Y) :-
	atom_length(X, 1), !,
	Y=X.
initial(X, Y) :-
	atomic_list_concat(Y1, '-', X),
	Y1 \= [X],
	!,
	atomic_list_concat(Y1, ' ', Z1),
	initials(Z1, Z2),
	atomic_list_concat(Z3, ' ', Z2),
	atomic_list_concat(Z3, '-', Y).
initial(X, Y) :-
	sub_atom(X, 0, 1, _, Y1),
	atomic_list_concat([Y1,'.'], Y).
