%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(textutilities,
	  [atom_to_ascii/2,
	   name_to_idlike_sortkey/2]).

atom_to_ascii(A, A1) :-
	atom_chars(A, Cs),
	map_aaa(Cs, Ca1),
	atom_chars(A1, Ca1).

name_to_idlike_sortkey(A, A1) :-
	downcase_atom(A, A2),
	atom_chars(A2, Cs),
	map_sss(Cs, Ca1),
	atom_chars(A1, Ca1).

map_aaa(X, Y) :-
	represent(X, Y, X1, Y1),
	!,
	map_aaa(X1, Y1).
map_aaa([X|Xs], [X|Xs1]) :-
	char_type(X, alnum),
	char_type(X, ascii),
	!,
	map_aaa(Xs, Xs1).
map_aaa([_|Xs], ['_'|Xs1]) :-
	map_aaa(Xs, Xs1).
map_aaa([], []).

map_sss(X, Y) :-
	represent(X, Y, X1, Y1),
	!,
	map_sss(X1, Y1).
map_sss([X|Xs], [X|Xs1]) :-
	char_type(X, alnum),
	char_type(X, ascii),
	!,
	map_sss(Xs, Xs1).
map_sss([',', ' '|Xs], [':'|Xs1]) :-
	!,
	map_sss(Xs, Xs1).
map_sss([_|Xs], ['_'|Xs1]) :-
	map_sss(Xs, Xs1).
map_sss([], []).

represent(['ä'|Xs], [a,e|Xs1], Xs, Xs1).
represent(['ö'|Xs], [o,e|Xs1], Xs, Xs1).
represent(['ü'|Xs], [u,e|Xs1], Xs, Xs1).
represent(['ß'|Xs], [s,s|Xs1], Xs, Xs1).
represent(['Ä'|Xs], ['A',e|Xs1], Xs, Xs1).
represent(['Ö'|Xs], ['O',e|Xs1], Xs, Xs1).
represent(['Ü'|Xs], ['U',e|Xs1], Xs, Xs1).
represent(['é'|Xs], [e|Xs1], Xs, Xs1).
represent(['ç'|Xs], [c|Xs1], Xs, Xs1).
represent(['ł'|Xs], [l|Xs1], Xs, Xs1).
represent(['æ'|Xs], [a,e|Xs1], Xs, Xs1).
represent(['œ'|Xs], [o,e|Xs1], Xs, Xs1).
represent(['É'|Xs], ['E'|Xs1], Xs, Xs1).
represent(['Ç'|Xs], ['C'|Xs1], Xs, Xs1).
represent(['Ł'|Xs], ['L'|Xs1], Xs, Xs1).
represent(['Æ'|Xs], ['A',e|Xs1], Xs, Xs1).
represent(['Œ'|Xs], ['O',e|Xs1], Xs, Xs1).

