%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(assistance, [main_context/1,
		       update_assistance/0,
		       ctx_get/2,
		       ctx_get_defaulted/3,
		       assistance_spec/1,
		       assistance_entry/1,
		       add_assistance_update_hook/1
% 		       def_context/2,
% 		       def_assistance/2,
% 		       make_effective/1
		       ]).

:- use_module(swilib(err)).
%%%% Assistance documents are loaded with Prolog's standard mechanisms

:- use_module(kbset_ner(ctx)).
:- use_module(swilib(info)).

:- dynamic loaded_assistance_document/1.
:- dynamic main_context/1.
:- dynamic assistance_spec/1.

:- multifile user:def_context/2.
:- multifile user:def_assistance/2.
:- multifile user:make_effective/1.

:- multifile user:dep/1.

:- multifile user:entity_features/2.
:- multifile user:entity_threshold_features/2.
:- multifile user:ner_processing_options/1.
:- multifile user:annotated_by/2.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
:- dynamic update_hook/1.

add_assistance_update_hook(Call) :-
	( update_hook(Call) -> true ;
	  assert(update_hook(Call))
	).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

register_assistance_document(Doc) :-
	loaded_assistance_document(Doc),
	!.
register_assistance_document(Doc) :-
	assert( loaded_assistance_document(Doc) ).

update_assistance :-
	info(10, 'Updating assistance'),
	( prolog_load_context(source, Doc) ->
	  register_assistance_document(Doc)
	; true
	),
	findall(PV, ( make_effective(Names),
		      member(Name, Names),
		      def_context(Name, DCTX),
		      member(PV, DCTX) ),
		DCTX1),
	make_ctx(DCTX1, CTX),
	retractall( main_context(_) ),
	assert( main_context(CTX) ),
	
	findall(Entry, ( make_effective(Names),
			 member(Name, Names),
			 def_assistance(Name, Ass),
			 member(Entry, Ass) ),
		Assistance),

	retractall( assistance_spec(_) ),
	assert( assistance_spec(Assistance) ),
	check_assistance,
	( update_hook(Call),
	  info(20, 'Calling hook ~w', [Call]),
	  Call,
	  fail
	; true
	).

ctx_get(X, Y) :-
	main_context(CTX),
	ctx_get(CTX, X, Y).

ctx_get_defaulted(X, Y, Z) :-
	main_context(CTX),
	ctx_get_defaulted(CTX, X, Y, Z).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

check_assistance :-
	( assistance_entry(_),
	  fail
	; true
	).

assistance_entry(Entry) :-
	assistance_spec(Spec),
	member(Entry1, Spec),
	canonicalize_assistance_entry(Entry1, Entry2),
	Entry = Entry2.


canonicalize_assistance_entry(supplement(Type, Spec, Sup), supplement(Type, Spec, Sup)) :-
	!.
canonicalize_assistance_entry(no_entity(Type, Where), no_entity(Type, Where)) :-
	!.
canonicalize_assistance_entry(entity(Type, Spec), entity(Type, Spec, [])) :-
	!.
canonicalize_assistance_entry(entity(Type, Spec, Where), entity(Type, Spec, Where)) :-
	!.
canonicalize_assistance_entry(X, _) :-
	err('Invalid assistance specification entry: ~q', [X]).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

time_of_last_update_assistance(X) :-
	flag(time_of_last_update_assistance, X, X).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- initialization(update_assistance).

