%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(ctx, [ make_ctx/1,
		 make_ctx/2,
		 ctx_get/3,
		 ctx_get_defaulted/4,
		 ctx_set/4,
		 ctx_set/3,
		 ctx_del/3,
		 ctx_get_multi/3,
		 ctx_add/4,
		 ctx_add/3
	       ]).

make_ctx(CTX) :-
	make_ctx([], CTX).

make_ctx(PVs, PVs).

ctx_get(CTX, P, V) :-
	memberchk(P=V1, CTX),
	!,
	V = V1.

ctx_get_multi(CTX, P, V) :-
	memberchk(P=V1, CTX),
	!,
	member(V, V1).

ctx_get_defaulted(CTX, P, V, Default) :-
	( memberchk(P=V1, CTX) ->
	  V = V1
	; V = Default
	).

ctx_del(CTX, P, CTX1) :-
	select(P=_, CTX, CTX1),
	!.
ctx_del(CTX, _, CTX).

ctx_set(CTX, P, V, CTX1) :-
	( memberchk(P=V1, CTX) ->
	  ( V = V1 ->
	    CTX = CTX1
	  ; select(P=_, CTX, CTX2),
	    !,
	    CTX1 = [P=V|CTX2]
	  )  
	; CTX1 = [P=V|CTX]
	).

ctx_add(CTX, P, V, CTX1) :-
	( memberchk(P=Vs, CTX) ->
	  ( memberchk(V, Vs) ->
	    CTX1 = CTX
	  ; select(P=Vs, CTX, CTX2),
	    !,
	    CTX1 = [P=[V|Vs]|CTX2]
	  )
	; CTX1 = [P=[V]|CTX]
	).

ctx_set(CTX, PVs, CTX1) :-
	ctx_set_1(PVs, CTX, CTX1).
ctx_set_1([], CTX, CTX).
ctx_set_1([P=V|PVs], CTX, CTX1) :-
	ctx_set(CTX, P, V, CTX2),
	ctx_set_1(PVs, CTX2, CTX1).

ctx_add(CTX, PVs, CTX1) :-
	ctx_add_1(PVs, CTX, CTX1).
ctx_add_1([], CTX, CTX).
ctx_add_1([P=V|PVs], CTX, CTX1) :-
	ctx_add(CTX, P, V, CTX2),
	ctx_add_1(PVs, CTX2, CTX1).
