%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(fmtutils, [format_header_1/1,
		     format_header_2/1,
		     write_members_separated_by_blanks/1,
		     write_single_line/0,
		     write_double_line/0,
		     indent/1]).

:- use_module(kbset_base(localization)).

format_header_2(X) :-
	write_single_line,
	ul_writeln(X),
	write_single_line.

format_header_1(X) :-
	write_double_line,
	ul_writeln(X),
	write_double_line.

ul_writeln(X) :-
	atom(X),
	!,
	writeln(X).
ul_writeln(Xs) :-
	user_language(Lang),
	memberchk(Lang-X, Xs),
	!,
	writeln(X).
ul_writeln([_-X|_]) :-
	writeln(X).

write_members_separated_by_blanks([A]) :-
	!, write(A).
write_members_separated_by_blanks([A|As]) :-
	write(A), write(' '),
	write_members_separated_by_blanks(As).
write_members_separated_by_blanks([]).


write_double_line :-
	write('=================================================='),
	writeln('============================').

write_single_line :-
	write('--------------------------------------------------'),
	writeln('----------------------------').

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

indent(N) :-
	N > 0,
	!,
	write(' '),
	N1 is N - 1,
	indent(N1).
indent(0).
