%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(latexannot,
	  [ file_to_abouts/2,
	    es_validate_abouts/2,
	    es_latannot_abouts/3,
	    es_latannot_persons/2,
	    es_latannot_locations/2,
	    es_latannot_dates/2,
	    es_latannot_quotes/2,
	    es_latannot_keywords/2]).

:- use_module(swilib(fromonto)).
:- use_module(swilib(err)).
:- use_module(swilib(term_handling)).
:- use_module(swilib(info)).

:- use_module(kbset_base(localization)).
:- use_module(kbset_base(utils)).
:- use_module(kbset_ner(esops)).
:- use_module(kbset_base(textutils)).
:- use_module(kbset_ner(layers)).
:- use_module(kbset_base(readwrite)).
:- use_module(kbset_ner(ner_utils)).
:- use_module(kbset_base(person)).
:- use_module(kbset_ner(date_ner)).
:- use_module(kbset_base(location)).

	
% tst0 :-
% %	F = '/Users/ch/tst/mini.tex',
% 	F='/Users/ch/p14/rea01/src/reaction_01_15.tex',
% 	doc_processed_es(F, Es),
% 	!,
% 	A='/Users/ch/p14/rea01/apparat/annot_reaction_01.tex',
% 	file_to_abouts(A, As),
% 	time(es_validate_abouts(Es, As)),
% 	time(es_latannot_abouts(Es, As, Es2)),
% 	time(es_latannot_dates(Es2, As, Es1)),
% 	es_to_file(Es1, '/Users/ch/tmp/tst02.tex').
% %		     es_latabout_persons(Es, Es1),
% %	es_to_file(Es1, '/Users/ch/tmp/tst01.tex').
% 
% 
% 
% tst :-
% %	F = '/Users/ch/tst/mini.tex',
% 	F='/Users/ch/p14/rea01/src/reaction_01_15.tex',
% 	doc_processed_es(F, Es),
% 	!,
% 	time(es_latannot_persons(Es, Es1)),
% 	es_to_file(Es1, '/Users/ch/tmp/tst01.tex').


es_latannot_persons(Es, Es1) :-
	info(10, 'Merging with person information'),
	es_latannot_flow(Es, person_margin_and_index, Es1).

es_latannot_locations(Es, Es1) :-
	info(10, 'Merging with location information'),
	es_latannot_flow(Es, location_margin_and_index, Es1).

es_latannot_abouts(Es, Abouts, Es1) :-
	info(10, 'Merging with \'about\' annotations'),
	es_latannot_flow(Es, about, Abouts, Es1).

es_latannot_dates(Es, Es1) :-
	info(10, 'Merging with date information'),
	es_latannot_flow(Es, date_margin_and_index, Es1).

es_latannot_keywords(Es, Es1) :-
	info(10, 'Merging with keyword index information'),
	es_latannot_flow(Es, kwd_index, Es1).

e_annot_elem(person_margin_and_index, E, _, _, _, EA) :-
	!,
	e_person_margin_and_index_elem(E, EA).

e_annot_elem(location_margin_and_index, E, _, _, _, EA) :-
	!,
	e_location_margin_and_index_elem(E, EA).

e_annot_elem(about, E, Es, Abouts, Abouts1, EA) :-
	!,
	e_process_abouts(Abouts, E, Es, Abouts1, EA).

e_annot_elem(date_margin_and_index, E, _, _, _, EA) :-
	!,
	e_date_margin_and_index_elem(E, EA).

e_annot_elem(kwd_index, E, _, _, _, EA) :-
	!,
	e_kwd_index_elem(E, EA).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% GENERIC CODE
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% 
%%%% Insert annotation elements before or after the scope of certain latex
%%%% commands, used e.g. for marginpar which are not allowed e.g. in chapter
%%%% headers and in footnotes
%%%%
es_latannot_flow(Es, Prop, Es1) :-	
	con_es_after_env_seq(Es, conaux_latannot,
			     k(Prop,Es1,B,B,D,D,_), k(_,C,C,D1,D1,[],_)).

es_latannot_flow(Es, Prop, Arg, Es1) :-	
	con_es_after_env_seq(Es, conaux_latannot,
			     k(Prop,Es1,B,B,D,D,Arg), k(_,C,C,D1,D1,[],_)).


insert_before_environment_command(foot).
insert_before_environment_command(xnote).
insert_before_environment_command(xfoot).
insert_before_environment_command(xsic).
insert_before_environment_command(xsicm).
insert_before_environment_command(xsicw).
%% OR UNDO textls OTHERWISE:
insert_before_environment_command(xo).
% insert_before_environment_command(bef).

insert_after_environment_command(xchapnc).
insert_after_environment_command(xchaph).
insert_after_environment_command(xchap).
insert_after_environment_command(xseca).
insert_after_environment_command(xsecn).
insert_after_environment_command(xsecnc).
% insert_after_environment_command(aft).

conaux_latannot(E, k(Prop,A,B,C,D,DR,X), k(Prop,A1,B1,C1,D,DR,X1), Es, Env) :-
	is_in_insert_before_environment(Env),
	!,
	C = [E|C1],
	( e_annot_elem(Prop, E, Es, X, X1, P) ->
	  A = [P|A1]
	; A = A1,
	  X1 = X
	),
	B1 = B.
conaux_latannot(E, k(Prop,A,B,C,D,DR,X), k(Prop,A1,B1,C1,D,DR1,X1), Es, Env) :-
	(( is_in_insert_after_environment(Env)
	 ; %% only insert stuff after first good word for marginfix to work
	   %% properly
	   \+ e_content(E, wrd(_))
	 )
	),
	!,
	A1 = A,
	B1 = B,
	C = [E|C1],
	( e_annot_elem(Prop, E, Es, X, X1, P) ->
	  DR = [P|DR1]
	; DR1 = DR,
	  X1 = X
	).
% %%%% 
% %%%% Insert immediately AFTER the E. Useful with marginfix, since marginpars
% %%%% immediately after \section are not handled properly if section starts at
% %%%% fresh page: the marginpar then might appear on the previous page.
% %%%% 

conaux_latannot(E, k(Prop,A,B,C,D,DR,X), k(Prop,A1,B1,C1,D1,D1,X1), Es, _Env) :-
	A = B,
	( e_annot_elem(Prop, E, Es, X, X1, P) ->
	  DR = [P|A1]
	; DR = A1,
	  X1 = X
	),
	C = [E|D],
	B1 = C1.

% conaux_latannot(E, k(Prop,A,B,C,D,DR,X), k(Prop,A1,B1,C1,D1,D1,X1), Es, _Env) :-
% 	writeq(now1(E, xa(A),xb(B),xc(C),xd(D),dr(DR))), nl,
% 	A=B,
% 	e_content(E, KKK),
% 	writeq(content(KKK)), nl,
% 	( e_annot_elem(Prop, E, Es, X, X1, P) ->
% 	  writeln('FOUND AT'),
% 	  C = [E,P|D],
% 	  DR = A1,
% 	; C = [E|D],
% 	  DR = A1,
% 	  X1 = X
% 	).
% v2
% conaux_latannot(E, k(Prop,A,A,C,C,DR,X), k(Prop,A1,B1,C1,D1,D1,X1), Es, _Env) :-
% 	e_content(E, KKK),
% 	writeq(content(KKK)), nl,
% 	( e_annot_elem(Prop, E, Es, X, X1, P) ->
% 	  writeln('FOUND AT'),
% 	  B1 = [E,P|C1],
% 	  DR = A1
% 	; B1 = [E|C1],
% 	  DR = A1,
% 	  X1 = X
% 	).

%%%%
%%%% Old version
%%%%
% conaux_latannot(E, k(Prop,A,B,C,D,DR,X), k(Prop,A1,B1,C1,D1,D1,X1), Es, _Env) :-
% 	A = B,
% 	( e_annot_elem(Prop, E, Es, X, X1, P) ->
% 	  DR = [P|A1]
% 	; DR = A1,
% 	  X1 = X
% 	),
% 	C = D,
% 	B1 = [E|C1].

is_in_insert_before_environment(Env) :-
	env_open_command(Env, Cmd),
	insert_before_environment_command(Cmd),
	!.
is_in_insert_after_environment(Env) :-
	env_open_command(Env, Cmd),
	insert_after_environment_command(Cmd),
	!.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% PERSON_MARGIN_AND_INDEX
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

e_person_margin_and_index_elem(E, EP) :-
	e_identified_item_id(persons, E, P1),
	!,
	person_pretty(P1, N0),
	esc_latex_pretty(N0, N),
	plain_sortkey(N, NK),
 	( person_url(P1, URL) ->
 	  esc_latex(URL, URL1),
 	  format(atom(LatexCode),
 		 '\\xmarginperson{\\href{~w}{~w}}\\index[per]{~w@\\href{~w}{~w}}',
		 [URL1, N, NK, URL1, N])
 	; format(atom(LatexCode),
 		 '\\xmarginperson{~w}\\index[per]{~w@~w}', [N, NK, N])
	  ),
	mk_e(opq(LatexCode), [], EP).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% LOCATION_MARGIN_AND_INDEX
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

e_location_margin_and_index_elem(E, EP) :-
	e_identified_item_id(locations, E, P1),
	!,
	location_pretty(P1, N0),
	esc_latex_pretty(N0, N),
	plain_sortkey(N, NK),
 	( location_url(P1, URL) ->
 	  esc_latex(URL, URL1),
 	  format(atom(LatexCode),
 		 '\\xmarginlocation{\\href{~w}{~w}}\\index[loc]{~w@\\href{~w}{~w}}',
		 [URL1, N, NK, URL1, N])
 	; format(atom(LatexCode),
 		 '\\xmarginlocation{~w}\\index[per]{~w@~w}', [N, NK, N])
	  ),
	mk_e(opq(LatexCode), [], EP).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% KWD_INDEX
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

e_kwd_index_elem(E, EP) :-
	e_get(E, kwd_index, NGrams),
	!,
	map_kwd_indexcode(NGrams, LatexCode),
	atomic_list_concat(LatexCode, LatexCode1),
	mk_e(opq(LatexCode1), [], EP).

map_kwd_indexcode([X|Xs], [X1|Xs1]) :-
	format(atom(X1), '\\index[kwd]{~w}', [X]),
	map_kwd_indexcode(Xs, Xs1).
map_kwd_indexcode([], []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% DATE_MARGIN_AND_INDEX
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

e_date_margin_and_index_elem(E, EP) :-
	e_get(E, date, begin(Date)),
	Date \= ignore-_,
	!,
	date_marginentry(Date, MDate),
	date_indexentry(Date, IDate),
	format(atom(LatexCode),
	       '\\xmargindate{~w}\\index[dat]{~w}',
	       [MDate, IDate]),
	mk_e(opq(LatexCode), [], EP).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% ABOUTS
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% 
%%%% 
% %%%%
% e_process_abouts(_, _, _, _, _) :-
% 	!,
% 	fail.

e_process_abouts([], _, _, _, _) :-
	!,
	fail.
e_process_abouts(Abouts, E, Es, Abouts1, EA) :-
	member(About, Abouts),
	About = about(_, PSpec, _),
	about_hit(PSpec, E, Es),
	!,
	e_process_abouts_1(Abouts, E, Es, Abouts1, TVs),
	about_values_to_e(TVs, EA).

e_process_abouts_1([About|Abouts], E, Es, Abouts1, [Type-Value|TVs]) :-
	About = about(Type, PSpec, Value),
	about_hit(PSpec, E, Es),
	!,
	( pspec_is_multi(PSpec) ->
	  Abouts1 = [About|Abouts2]
	; Abouts2 = Abouts1
	),
	e_process_abouts_1(Abouts, E, Es, Abouts2, TVs).
e_process_abouts_1([About|Abouts], E, Es, [About|Abouts1], TVs) :-
	e_process_abouts_1(Abouts, E, Es, Abouts1, TVs).
e_process_abouts_1([], _, _, [], []).

about_hit(PSpec, E, Es) :-
	e_content(E, wrd(W)),
	pspec_txt(PSpec, W, Ws),
	es_word_prefix(Ws, Es).

about_values_to_e(TCs, EA) :-
	about_values_to_e_1(TCs, Codes),
	atomic_list_concat(Codes, Code),
	mk_e(opq(Code), [], EA).

about_values_to_e_1([Type-Code|TCs], [Code1|Codes]) :-
	format(atom(Code1), '\\XABOUT{~w}{~w}', [Type, Code]),
	about_values_to_e_1(TCs, Codes).
about_values_to_e_1([], []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% 
%%%% 
%%%% 
es_validate_abouts(Es, Abouts) :-
	info(10, 'Validating position specifiers of \'about\' annotations'),
	( member(about(_, PSpec, _), Abouts),
	  validate_pspec(PSpec, Es),
	  fail
	; true
	).

validate_pspec(PSpec, _) :-
	pspec_is_multi(PSpec),
	!.
validate_pspec(PSpec, Es) :-
	( mem([E|Es1], Es), about_hit(PSpec, E, Es1) ->
	  ( pspec_is_first(PSpec) ->
	    true
	  ; ( mem([E2|Es2], Es1), about_hit(PSpec, E2, Es2) ->
	      err('Ambiguous position specifier: ~q', [PSpec])
	    ; true
	    )
	  )
	; err('Void position specifier: ~q', [PSpec])
	).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- initialization(register_latex_command(xabout, [c,c,c], '')).

%%%% 
%%%% 
%%%% 
file_to_abouts(File, Abouts) :-
	info(10, 'Loading \'about\' annotations ~w', [File]),
	file_to_es(File, Es),
	es_to_abouts(Es, Abouts).

es_to_abouts(Es, [A|As]) :-
	mem([E|Es1], Es),
	e_content(E, cmd(xabout, [c,c,c])),
	!,
	es_ctrs([C1,C2,C3], Es1, Es2),
	canonicalize_position_spec(C2, C21),
	A = about(C1, C21, C3),
	es_to_abouts(Es2, As).
es_to_abouts(_, []).

es_ctrs([C|Cs], Es, Es1) :-
	mem([E|Es2], Es),
	e_content(E, ctr(C)),
	!,
	es_ctrs(Cs, Es2, Es1).
es_ctrs([], Es, Es) :-
	!.
es_ctrs(_, _, _) :-
	err('Missing arguments of \\xabout.').

pspec_is_multi(pspec_txt_multi(_, _)).
pspec_is_first(pspec_txt_first(_, _)).

pspec_txt(pspec_txt(W,Ws), W, Ws) :- !.
pspec_txt(pspec_txt_multi(W,Ws), W, Ws).
pspec_txt(pspec_txt_first(W,Ws), W, Ws).

canonicalize_position_spec(X, Y) :-
	atomic_list_concat(['[', X, ']'], X1),
	term_atom(Y1, X1),
	map_canonicalize_position_spec_elem(Y1, Y2),
	( Y2 = [txt=[W|Ws]] ->
	  Y = pspec_txt(W, Ws)
	; Y2 = [txt=[W]] ->
	  Y = pspec_txt(W, [])
	; Y2 = [txt_multi=[W|Ws]] ->
	  Y = pspec_txt_multi(W, Ws)
	; Y2 = [txt_multi=[W]] ->
	  Y = pspec_txt_multi(W, [])
	; Y2 = [txt_first=[W|Ws]] ->
	  Y = pspec_txt_first(W, Ws)
	; Y2 = [txt_first=[W]] ->
	  Y = pspec_txt_first(W, [])
	; err('Unsupported position specifier: ~q', [X])
	).

canonicalize_position_spec_elem(txt=Txt, txt=Ws) :-
	atom(Txt),
	!,
	split_atom_at_punct_and_space(Txt, Ws).
canonicalize_position_spec_elem(txt_multi=Txt, txt_multi=Ws) :-
	atom(Txt),
	!,
	split_atom_at_punct_and_space(Txt, Ws).
canonicalize_position_spec_elem(txt_first=Txt, txt_first=Ws) :-
	atom(Txt),
	!,
	split_atom_at_punct_and_space(Txt, Ws).
canonicalize_position_spec_elem(X, X).

map_canonicalize_position_spec_elem([X|Xs], [X1|Xs1]) :-
	canonicalize_position_spec_elem(X, X1),
	map_canonicalize_position_spec_elem(Xs, Xs1).
map_canonicalize_position_spec_elem([], []).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


es_latannot_quotes(Es, Es1) :-
	info(10, 'Merging with detected quoting'),
	map_e_latannot_quotes(Es, Es1).

map_e_latannot_quotes([E|Es], Es1) :-
	( e_get(E, quotes, Q) ->
	  ( Q = [begin(_)] ->
	    mk_e(opq('\\begin{XQUOTE}'), [], EX),
	    Es1 = [EX,E|Es2]
	  ; Q = [end(_)] ->
	    mk_e(opq('\\end{XQUOTE}'), [], EX),
	    Es1 = [E,EX|Es2]
	  )
	; Es1 = [E|Es2]
	),
	map_e_latannot_quotes(Es, Es2).
map_e_latannot_quotes([], []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% format('~|~`0t~d~2+', [2]).


date_marginentry(D-_, X) :-
	!,
	date_marginentry(D, X).

date_marginentry(y(Y), Y) :-
	!.
date_marginentry(ym(Y,M), I) :-
	!,
	month_pretty(M, M1),
	format(atom(I), '~w ~w', [M1, Y]).
date_marginentry(ymd(Y,M,D), I) :-
	!,
	month_pretty(M, M1),
	day_of_week(Y, M, D, DOW),
	format(atom(I), '~|~`0t~d~2+. ~w ~w, ~w', [D, M1, Y, DOW]).
date_marginentry(md(M,D), I) :-
	!,
	month_pretty(M, M1),
	format(atom(I), '~w. ~w ????', [D, M1]).
date_marginentry(y(Y,A), I) :-
	!,
	format(atom(I), '~w ~w', [A, Y]).
date_marginentry(a(A), A) :-
	!.
date_marginentry(X, I) :-
	format(atom(I), '~w', X).

date_indexentry(D-_, X) :-
	!,
	date_indexentry(D, X).

date_indexentry(y(Y), I) :-
	!,
	format(atom(I), '~|~`0t~d~4+@~w',
	       [Y,Y]).
date_indexentry(ym(Y,M), I) :-
	!,
	month_pretty(M, M1),
	format(atom(I), '~|~`0t~d~4+@~w!~|~`0t~d~2+@~w',
	       [Y,Y,M,M1]).
% date_indexentry(ymd(Y,M,D), I) :-
% 	!,
% 	month_pretty(M, M1),
% 	format(atom(I), '~|~`0t~d~4+@~w!~|~`0t~d~2+~|~`0t~d~2+@~|~`0t~d~2+. ~w',
% 	       [Y,Y,M,D,D,M1]).
date_indexentry(ym(Y,M), I) :-
	!,
	month_pretty(M, M1),
	format(atom(I), '~|~`0t~d~4+@~w!~|~`0t~d~2+@~w',
	       [Y,Y,M,M1]).
date_indexentry(ymd(Y,M,D), I) :-
	!,
	month_pretty(M, M1),
	format(atom(I), '~|~`0t~d~4+@~w!~|~`0t~d~2+@~w!~|~`0t~d~2+@~|~`0t~d~2+.',
	       [Y,Y,M,M1,D,D]).
date_indexentry(md(M,D), I) :-
	!,
	month_pretty(M, M1),
	format(atom(I), '????!~|~`0t~d~2+@~w!~|~`0t~d~2+@~|~`0t~d~2+.',
	       [M,M1,D,D]).
date_indexentry(y(Y,A), I) :-
	!,
	format(atom(I), '~|~`0t~d~4+@~w!~w',
	       [Y,Y,A]).
date_indexentry(a(A), A) :-
	!.
date_indexentry(X, I) :-
	format(atom(I), '~w', X).

month_pretty(X, Y) :-
	user_language(Lang),
	month_pretty(Lang, X, Y).

month_pretty(de, X, Y) :-
	!,
	month_pretty_de(X, Y).
month_pretty(en, X, Y) :-
	!,
	month_pretty_en(X, Y).
month_pretty(_, X, X).

month_pretty_de(1, 'Januar').
month_pretty_de(2, 'Februar').
month_pretty_de(3, 'März').
month_pretty_de(4, 'April').
month_pretty_de(5, 'Mai').
month_pretty_de(6, 'Juni').
month_pretty_de(7, 'Juli').
month_pretty_de(8, 'August').
month_pretty_de(9, 'September').
month_pretty_de(10, 'Oktober').
month_pretty_de(11, 'November').
month_pretty_de(12, 'Dezember').

month_pretty_en(1, 'January').
month_pretty_en(2, 'February').
month_pretty_en(3, 'March').
month_pretty_en(4, 'April').
month_pretty_en(5, 'May').
month_pretty_en(6, 'June').
month_pretty_en(7, 'July').
month_pretty_en(8, 'August').
month_pretty_en(9, 'September').
month_pretty_en(10, 'October').
month_pretty_en(11, 'November').
month_pretty_en(12, 'December').
