%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2016 , 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(ner, [es_ner/3,
		ner_show_es_item_summary/3,
		ner_show_es_date_summary/1,
		ner_inspect_es_item_at_srcpos/2
	       ]).

% :- use_module(ner_old).
:- use_module(kbset_ner(layers)).
:- use_module(kbset_ner(ner_base)).
:- use_module(kbset_ner(ner_utils)).
:- use_module(kbset_base(person)).
:- use_module(kbset_base(location)).
:- use_module(kbset_ner(person_ner)).
:- use_module(kbset_ner(location_ner)).
:- use_module(kbset_ner(date_ner)).
:- use_module(kbset_ner(detect)).
:- use_module(kbset_ner(assistance)).
:- use_module(kbset_base(localization)).
:- use_module(kbset_ner(fmtutils)).
:- use_module(kbset_base(utils)).
:- use_module(swilib(info)).
:- use_module(kbset_base(dates)).
:- use_module(kbset_ner(ctx)).
:- use_module(swilib(err)).

es_ner(Es, Options, Es1) :-
	info(10, 'Entity recognition with options ~q', [Options]),
	main_context(CTX),
	( memberchk(quotes, Options) ->
	  es_set_quotes(Es, Es10),
	  CTX10 = CTX
	; Es10 = Es, CTX10 = CTX
	),
	( memberchk(dates, Options) ->
	  es_add_parsed_dates_german(Es10, Es11),
	  time(es_add_dates(Es11, CTX10, Es12, CTX12))
	; Es12 = Es10, CTX12 = CTX10
	),
	( memberchk(function_persons, Options) ->
	  time(es_add_function_persons(Es12, CTX12, Es13, CTX13))
	; Es13 = Es12, CTX13 = CTX12
	),
	( memberchk(persons, Options) ->
	  time(es_add_persons(Es13, CTX13, Es14, CTX14))
	; Es14 = Es13, CTX14 = CTX13
	),
	( memberchk(locations, Options) ->
	  time(es_add_locations(Es14, CTX14, Es15, _CTX15))
	; Es15 = Es14
	),
	Es1 = Es15.

%
% old:	
	
% in a loop:	
% 	( ctx_get(CTX2, modified, true) ->
% 	  es_add_persons(Es2, CTX2, Es1, CTX1)
% 	; Es1 = Es2,
% 	  CTX1 = CTX2
% 	).

:- flag(cnt, _, 0).


es_add_persons(Es, CTX, Es1, CTX1) :-
	length(Es, Num),
	flag(cnt, _, 0),
	info(10, 'Scanning for persons (~D elements)', [Num]),
	ctx_set(CTX, identified_items, [], CTX2),
	map_outrevbefore_after(e_add_persons, Es, CTX2, CTX1, Es1),
	info_progress_done.

es_add_function_persons(Es, CTX, Es1, CTX1) :-
	length(Es, Num),
	flag(cnt, _, 0),
	ctx_set(CTX, identified_items, [], CTX2),
	info(10, 'Scanning for function persons (~D elements)', [Num]),
	map_outrevbefore_after(e_add_function_persons, Es, CTX2, CTX1, Es1),
	info_progress_done.

es_add_locations(Es, CTX, Es1, CTX1) :-
	length(Es, Num),
	flag(cnt, _, 0),
	info(10, 'Scanning for locations (~D elements)', [Num]),
	ctx_set(CTX, identified_items, [], CTX2),
	map_outrevbefore_after(e_add_locations, Es, CTX2, CTX1, Es1),
	info_progress_done.

e_add_locations(E, CTX, CTX1, RevBefore, After, E1) :-
	e_add_items(location, locations, E, CTX, CTX1, RevBefore, After, E1).
e_add_persons(E, CTX, CTX1, RevBefore, After, E1) :-
	e_add_items(person, persons, E, CTX, CTX1, RevBefore, After, E1).
e_add_function_persons(E, CTX, CTX1, RevBefore, After, E1) :-
	e_add_items(function_person, persons, E, CTX, CTX1, RevBefore, After, E1).

e_add_items(EntityProp, ElemProp, E, CTX, CTX1, RevBefore, After, E1) :-
	flag(cnt, N, N+1),
	info_progress(10, 1000, N),
	ctx_set(CTX, [elems_rev_before=RevBefore,
		      elems_after=After], CTX2),
	( ctx_get(CTX2, ner_lock_until, Until),
	  e_content(E, Until) ->
	  e_set(E, ner_locked, true, E1),
	  ctx_del(CTX2, ner_lock_until, CTX1)
	; e_get(E, ner_locked, true) ->
	  E1 = E, CTX1 = CTX2		
	; e_entities(E, EntityProp, CTX2, Ps),
	  ( Ps = [] ->
	    E1 = E,
	    CTX1 = CTX2
	  ; e_set(E, ElemProp, Ps, E1),
	    ctx_set(CTX2, modified, true, CTX3),
	    %% Only lock subsequent if the item is IDENTIFIED (the lock
	    %% must be wrt a single identified entity)
	    ( e_identified_item(ElemProp, E1, IdentifiedItem) ->
	      ner_item_id(IdentifiedItem, IdItem),
	      ctx_get(CTX3, identified_items, IdItems),
	      %% IdItems is in reverse order of occurrence, with duplicates
	      ctx_set(CTX3, identified_items, [IdItem|IdItems], CTX4),
	      ner_item_features(IdentifiedItem, Features),
	      ( features_lock_until(EntityProp, Features, Until) ->
		ctx_set(CTX4, ner_lock_until, Until, CTX1)
	      ; CTX1 = CTX4
	      )
	    ; CTX1 = CTX3
	    )
	  )
	).

es_add_dates(Es, CTX, Es1, CTX1) :-
	length(Es, Num),
	flag(cnt, _, 0),
	info(10, 'Processing dates (~D elements)', [Num]),
	date_ner_init_ctx(CTX, CTX2),
	map_outrevbefore_after(e_add_dates, Es, CTX2, CTX1, Es1),
	info_progress_done.

e_add_dates(E, CTX, CTX1, _RevBefore, After, E1) :-
	flag(cnt, N, N+1),
	info_progress(10, 1000, N),
	ctx_set(CTX, [elems_after=After], CTX2),
	e_entities(E, date, CTX2, Ds),
	( Ds = [] ->
	  E1 = E,
	  CTX1 = CTX2
	; Ds = [begin(D)-R|_] ->
	  e_set(E, date, begin(D-R), E1),
	  date_ner_update_ctx(CTX2, D, CTX3),
	  ctx_set(CTX3, modified, true, CTX1)
	; Ds = [begin_end(D)-R|_] ->
	  e_set(E, date, begin_end(D-R), E1),
	  date_ner_update_ctx(CTX2, D, CTX3),
	  ctx_set(CTX3, modified, true, CTX1)
	; Ds = [end-_|_] ->
	  e_set(E, date, end, E1),
	  CTX1 = CTX2
	).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% 
%%%%

type_prop(person, persons).
type_prop(location, locations).

show_item_nonl(person, X) :- !, show_person_nonl(X).
show_item_nonl(location, X) :- !, show_location_nonl(X).
show_item_vwr(person, X) :- !, show_person_vwr(X).
show_item_vwr(location, X) :- !, show_location_vwr(X).
show_item_vw(person, X) :- !, show_person_vw(X).
show_item_vw(location, X) :- !, show_location_vw(X).

write_item_summary_header(Type, NoO, NoE) :-
	user_language(Lang),
	write_item_summary_header(Lang, Type, NoO, NoE).

write_item_summary_header(de, Type, NoO, NoE) :-
	!,
	write('* Zusammenfassung: '),
	( Type = person ->
	  write('Personen')
	; Type = location ->
	  write('Orte')
	; write(Type)
	),
	format(', ~w Vorkommen, ~w Kandidaten-Objekte *~n', [NoO, NoE]).
write_item_summary_header(_, Type, NoO, NoE) :-
	!,
	write('* Summary: '),
	( Type = person ->
	  write('Persons')
	; Type = location ->
	  write('Locations')
	; write(Type)
	),
	format(' ~w Occurrences, ~w Candidate entities~n', [NoO, NoE]).	

ner_show_es_item_summary(Type, Es, Options) :-
	type_prop(Type, Prop),
	findall(t(Item,Word,Status,Rationale),
		( member(E, Es),
		  e_get(E, Prop, Ps1),
		  Ps1 \= [],
		  ( e_content(E, wrd(Word)) -> true ; Word = '' ),
		  sort_entities(Type, Ps1, keep_best_group, Ps),
		  ( Ps = [_] -> Status = identified ; Status = ambiguous ),
		  member(Item-Rationale, Ps)
		),
		Ks),
	msort(Ks, Ks1),
	( memberchk(sort=alpha, Options) ->
	  nseis_gbf_1_alpha(Ks1, Gs)
	; nseis_gbf_1(Ks1, Gs)
	),
	length(Ks1, NOccs),
	length(Gs, NIds),
	write_item_summary_header(Type, NOccs, NIds),
	( member(G, Gs),
	  nseis_show_item_group(Type, G, Options),
	  fail
	; true
	).

localized(X, Y) :-
	user_language(Lang),
	localized(Lang, X, Y),
	!.
localized(X, X).

localized(de, occurrences, 'Vorkommen').
localized(en, occurrences, 'occurrences').
localized(de, ambiguous, 'uneindeutig').
localized(en, ambiguous, 'ambiguous').

nseis_show_item_group(Type, i(J,_N,Ks), Options) :-
	write_double_line,
	nseis_mw_1(Ks, Gs),
	write('* '),
	( member(G, Gs),
	  nseis_show_word_group(Type, G, Options),
	  fail
	; nl
	),
	write_single_line,
	show_item_vw(Type, J),
	( memberchk(show_rationales=true, Options) ->
	  findall(R, (member(p(_, _, Ks1), Gs), member(k(_,R), Ks1)), Rs),
	  sort(Rs, Rs1),
	  ( member(pro(_,R1), Rs1),
	    write_single_line,
	    show_reasons(R1),
	    fail
	  ; true
	  )
	; true
	).
	    
nseis_show_word_group(_Type, p(W,N,Ks), _Options) :-
	format('~w (~w', [W, N]),
	( memberchk(k(ambiguous,_), Ks) ->
	  localized(ambiguous, Ambiguous),
	  format(', ~w) ', [Ambiguous])
	; format(') ')
	).

nseis_mw_1(Ks, Gs) :-
	msort(Ks, [k(W,S,R)|Ks1]),
	nseis_mw_2(Ks1, W, 1, [k(S,R)], Gs1),
	keysort(Gs1, Gs2),
	map_val(Gs2, Gs).

nseis_mw_2([k(W,S,R)|Ks], W, N, G, Gs) :-
	!,
	N1 is N + 1,
	nseis_mw_2(Ks, W, N1, [k(S,R)|G], Gs).
nseis_mw_2([k(W,S,R)|Ks], V, N, G, [Key-p(V,N,G1)|Gs]) :-
	Key is 1/(N+1),
	sort(G, G1),
	nseis_mw_2(Ks, W, 1, [k(S,R)], Gs).
nseis_mw_2([], V, N, G, [Key-p(V,N,G1)]) :-
	Key is 1/(N+1),
	sort(G, G1).


nseis_gbf_1([t(I,W,S,R)|Ks], Gs) :-
	nseis_gbf_2(Ks, I, 1, W, [k(W,S,R)], Gs1),
	keysort(Gs1, Gs2),
	map_val(Gs2, Gs).
	
nseis_gbf_2([t(I,W,S,R)|Ks], I, N, V, G, Gs) :-
	!,
	N1 is N + 1,
	nseis_gbf_2(Ks, I, N1, V, [k(W,S,R)|G], Gs).
nseis_gbf_2([t(I,W,S,R)|Ks], J, N, V, G, [Key-i(J,N,G)|Gs]) :-
	OccKey is 1/(N+1),
	Key = key(OccKey,V),
	nseis_gbf_2(Ks, I, 1, W, [k(W,S,R)], Gs).
nseis_gbf_2([], J, N, V, G, [Key-i(J,N,G)]) :-
	OccKey is 1/(N+1),
	Key = key(OccKey,V).


nseis_gbf_1_alpha([t(I,W,S,R)|Ks], Gs) :-
	nseis_gbf_2_alpha(Ks, I, 1, W, [k(W,S,R)], Gs1),
	keysort(Gs1, Gs2),
	map_val(Gs2, Gs).

nseis_gbf_2_alpha([t(I,W,S,R)|Ks], I, N, V, G, Gs) :-
	!,
	N1 is N + 1,
	nseis_gbf_2_alpha(Ks, I, N1, V, [k(W,S,R)|G], Gs).
nseis_gbf_2_alpha([t(I,W,S,R)|Ks], J, N, V, G, [Key-i(J,N,G)|Gs]) :-
	Key = key(V),
	nseis_gbf_2_alpha(Ks, I, 1, W, [k(W,S,R)], Gs).
nseis_gbf_2_alpha([], J, N, V, G, [Key-i(J,N,G)]) :-
	Key = key(V).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

local_map_key([X-_|Xs], [X|Xs1]) :-
	local_map_key(Xs, Xs1).
local_map_key([], []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% 
%%%% Parsed Dates
%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% 
%%%% 
%%%%
es_add_parsed_dates_german(Es, Es1) :-
	info(10, 'Adding parsed dates'),
	es_add_parsed_dates(de, Es, Es1).

es_add_parsed_dates(Language, Es, Es1) :-
	info(50, 'Parsing dates'),
	findall(X, aux_es_parsed_date_spec(Language, Es, X), Xs),
	length(Xs, L),
	L1 is L div 2,
	info(50, 'Found ~w dates', [L1]),
	info(50, 'Merging in'),
	es_mergein(Xs, Es, Es1).

aux_es_parsed_date_spec(Language, Es, S) :-
	es_plain_codes(Es, Cs),
	( Language = de ->
	  de_enum_date(Cs, D, P1, P2)
	; err('No language support for date parsing for: ~q', [Language])
	),
	( mk_e(nil, [parsed_date=begin(D)], E),
	  S = before(P1, E)
	; mk_e(nil, [parsed_date=end], E),
	  S = behind(P2, E)
	).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


ner_show_es_date_summary(Es) :-
	format('*DATE SUMMARY*~n'),
	( member(E, Es),
	  ( (e_get(E, date, begin(D)) ; e_get(E, date, begin_end(D))) ->
	    D \= ignore-_,
	    write('--------------------------------------------------'),
	    writeln('----------------------------'),
	    show_date_vwr(D)
	  ; true
	  ),
	  fail
	; true
	).

% ner_show_es_date_summary(Es) :-
% 	format('*DATE SUMMARY*~n'),
% 	( member(E, Es),
% 	  ( e_get(E, parsed_date, begin(D)) ->
% 	    date_normal(D, ND),
% 	    format('Parsed: ~w~n', [ND])
% 	  ; true
% 	  ),
% 	  fail
% 	; true
% 	).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


ner_inspect_es_item_at_srcpos(Es, SrcPos) :-
	main_context(CTX),
	ctx_set(CTX, [current_pos=0, required_pos=SrcPos], CTX1),
	catch( map_outrevbefore_after(inspect_item_at_srcpos, Es, CTX1, _, _),
	       inspect_item_at_srcpos_exception(E, CTX2),
	       ner_show_inspect_item(E, CTX2) ).

ner_show_inspect_item(E, CTX) :-
	write_inspect_header(Lang, srctxt),
	write('"'),
	( e_enum_srctxt(E, Txt), write(Txt), fail ; writeln('"') ),
	user_language(Lang),
	write_inspect_header(Lang, persons),
	e_entities_ignore_threshold(E, person, keep_all, CTX, Persons),
	show_persons_vwr(Persons),
	write_inspect_header(Lang, function_persons),
	e_entities_ignore_threshold(E, function_person, keep_all, CTX, FunctionPersons),
	show_persons_vwr(FunctionPersons),
	e_entities_ignore_threshold(E, location, keep_all, CTX, Locations),
	write_inspect_header(Lang, locations),
	show_locations_vwr(Locations).

write_inspect_header(Lang, Key) :-
	write_double_line,
	( inspect_header(Lang, Key, Header) ->
	  writeln(Header)
	; writeln(Key)
	),
	write_double_line.

inspect_header(de, srctxt, 'Quelltext').
inspect_header(de, persons, 'Personen').
inspect_header(de, function_persons,
	       'Über Funktion charakterisierte Personen').
inspect_header(de, locations, 'Orte').
inspect_header(de, srctxt, 'Source Text').
inspect_header(en, personen, 'Persons').
inspect_header(en, function_persons,
	       'Persons Characterized by Function').
inspect_header(en, locations, 'Locations').

	
inspect_item_at_srcpos(E, CTX, CTX1, RevBefore, After, E) :-
	ctx_get(CTX, required_pos, Pos),
	ctx_get(CTX, current_pos, CPos),
	e_srctxt_size(E, S),
	CPos1 is CPos + S,
	ctx_set(CTX, [current_pos=CPos1,
		      elems_rev_before=RevBefore,
		      elems_after=After], CTX1),
	( CPos1 >= Pos ->
	  throw(inspect_item_at_srcpos_exception(E, CTX1))
	; true
	).

