%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2014, 2015, 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- module(wordutils,
	  [word_is_capitalized/1,
	   word_is_common_substantive/2,
	   word_is_common_geoname/2,
	   word_is_countryname/1,
	   word_is_common_downcase/2,
	   register_common_substantive/1,
	   register_common_downcase/1]).
	   
:- use_module(kbset_base(geo)).
:- use_module(kbset_ner(germanutils)).
:- use_module(kbset_base(textutils)).
:- use_module(kbset_data(wordlists_de)).
:- use_module(kbset_base(location)).

:- dynamic ext_common_substantive/1.
:- dynamic ext_common_downcase/1.

register_common_substantive(X) :-
	( ext_common_substantive(X) ->
	  true
	; assert( ext_common_substantive(X) )
	).

register_common_downcase(X) :-
	( ext_common_downcase(X) ->
	  true
	; assert( ext_common_downcase(X) )
	).

word_is_common_downcase(Lang, W) :-
	downcase(W, W1),
	word_is_common_downcase_1(Lang, W1).

word_is_common_downcase_1(_, W) :-
	ext_common_downcase(W),
	!.
word_is_common_downcase_1(de(19), W) :-
	!,
	word_is_common_downcase_de_19(W).	
word_is_common_downcase_1(de(_), W) :-
	!,
	lowercase_de(W).

word_is_common_downcase_de_19(W) :-
	lowercase_de(W),
	!.
word_is_common_downcase_de_19(W) :-
	de_nineteenth_tentative_renewed(W, W1),
	lowercase_de(W1),
	!.

%% todo: country names with blanks
%%
word_is_countryname(W) :-
	country_name(W).
	
word_is_capitalized(W) :-
	sub_atom(W, 0, 1, _, C1),
	is_upper_char(C1),
	( sub_atom(W, 1, 1, _, C2) ->
	  \+ is_upper_char(C2)
	; true
	).

word_is_common_substantive(_, W) :-
	ext_common_substantive(W),
	!.
word_is_common_substantive(de(19), W) :-
	!,
	word_is_common_substantive_de_19(W).
word_is_common_substantive(de(_), W) :-
	!,
	word_is_common_substantive_de(W).

word_is_common_substantive_de(W) :-
 	substantive_de(W),
 	!.

word_is_common_substantive_de_19(W) :-
	word_is_common_substantive_de(W),
	!.
word_is_common_substantive_de_19(W) :-
	de_nineteenth_tentative_renewed(W, W1),
	word_is_common_substantive_de(W1),
	!.

word_is_common_geoname(_, W) :-
	country_name(W),
	!.
word_is_common_geoname(_, W) :-
	geo_name_to_id(W, Y),
	( geo_city(Y, 50000, _, _),
	  geo_has_url(Y) ->
	  !
	; location_population(Y, P),
	  P > 50000,
	  !
	),
	!.
word_is_common_geoname(de, W) :-
	sub_atom(W, _, 1, 0, s),
	\+ sub_atom(W, _, 1, 1, s),
	sub_atom(W, 0, _, 1, W1),
	word_is_common_geoname(de, W1).


% word_is_common_geoname(_, W) :-
% 	geo_name_to_id(W, Y),
% 	geo_city(Y, 50000, _, _),
% 	!.
% word_is_common_geoname(_, W) :-
% 	geo_name_to_id(W, Y),
% 	geo_city(Y, 250000, _, _),
% 	!.

