%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%
%%%% Copyright (C) 2019 Christoph Wernhard
%%%%
%%%% This file is part of KBSET.
%%%%
%%%% KBSET is free software: you can redistribute it and/or modify
%%%% it under the terms of the GNU General Public License as published by
%%%% the Free Software Foundation, either version 3 of the License, or
%%%% (at your option) any later version.
%%%% 
%%%% KBSET is distributed in the hope that it will be useful,
%%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%%%% GNU General Public License for more details.
%%%% 
%%%% You should have received a copy of the GNU General Public License
%%%% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% 
%%%% A lightweight version for loading KBSET, which ensures that modules
%%%% required for letter editions (but not the full NER functionality) are
%%%% present. 
%%%%

:-	prolog_load_context(directory, LoadDir),
	( member(Part,
		 [swilib, kbset_base, kbset_letters, kbset_ner, kbset_data]),
	  atomic_list_concat([LoadDir, '/', Part], Path),
	  \+ user:file_search_path(Part, Path),
	  asserta(user:file_search_path(Part, Path)),
	  fail
	; true
	).

:- encoding(utf8).
:- set_prolog_flag(encoding, utf8).
:- set_stream(user_input, encoding(utf8)).
:- set_stream(user_output, encoding(utf8)).
:- set_stream(user_error, encoding(utf8)).

:- set_prolog_flag(qcompile, large).
:- set_prolog_flag(toplevel_print_anon, false).
:- set_prolog_flag(verbose, true).
% :- set_prolog_flag(verbose_load, true).

:- use_module(swilib(pretty)).
:- use_module(swilib(fromonto)).
:- use_module(swilib(info)).
:- use_module(kbset_letters(letters_interface)).

:- set_info_verbosity(10).
